﻿using System;
using System.Data;
using System.Text;
using System.Globalization;
using System.Reflection;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using System.Collections.Generic;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Qspac.Generators.BarcodeController;
using Produmex.Sbo.Logex.Qspac.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Qspac.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;


// PmxWorkflowExecutionTypes.SUB_FLOW
namespace WorkflowScript_QuantityCalculationQspac
{
    /// <summary>
    /// QSpac
    /// </summary>
    public class WorkflowScript_QuantityCalculationQspac : WorkflowInstanceScriptBase
    {
        /// <summary>
        /// Definitions
        /// </summary>
        public static class Definitions
        {
			/// <summary>
			/// Itemgroup splitter
			/// </summary>
			public const char ITEMGROUP_SPLITTER = '|';
		}

        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;


		//Output parameters
		public ReadWriteBinder<Dictionary<string, double>> ItemList;

		ISboProviderService m_sboProviderService;

		


		/// <summary>
		/// Initializes a new instance of the <see cref="WorkflowScript_QuantityCalculationQspac"/> class.
		/// </summary>
		/// <param name="parent">The parent.</param>
		/// <param name="factory">The factory.</param>
		public WorkflowScript_QuantityCalculationQspac(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
         
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
			List<string> itemGroups = new List<string>();
			Dictionary<string, PmxParameterValue> bcParameters = new Dictionary<string, PmxParameterValue>();
			Dictionary<string, double> itemList = new Dictionary<string, double>();
		

			string itemGroup = null;
			DataSet ds = null;

			// Parameters in scope
			Session session = GetScopeParameter("Session") as Session;
            m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;                 
            
            SboUserInfo userInfo = m_sboProviderService.GetCurrentUser();

			QSPACBarcodeController barcodeController = null;
			
            m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
            delegate (PmxDbConnection conn, object[] parameters)
            {
                PmxBarcodeProvider bcProv = new PmxBarcodeProvider(conn);
                barcodeController = (QSPACBarcodeController) bcProv.GetBarcodeController();
				bcParameters = bcProv.GetParametersBarcodeController();
				return null;
            });


		
			itemGroup = PmxParameterValueProvider.GetParameterValue<string>(bcParameters, PmxQSPExtensionParameterCodes.ITEM_GROUPS_FOR_QUANTITY_CALCULATION, "");	
			itemGroups = new List<string>(itemGroup.Split(Definitions.ITEMGROUP_SPLITTER));

			if (itemGroups.Count > 0)
			{

				string itemCode = null;
				double width = 0;
				double numInBuy = 0;
				double value = 0;

				string query = BuildQuery.GetItemsInfo(itemGroups, m_sboProviderService.GetDbTool());							
				ds = m_sboProviderService.RunView(false, null, null, query);

				//Get info for all items of these item groups
				if (this.GetNumberOfRows(ds) > 0)
				{
					foreach (DataRow row in ds.Tables[0].Rows)
					{
						 itemCode = Convert.ToString(row[PmxItemTable.Columns.ItemCode.NAME]);

						 numInBuy = Convert.ToDouble(row[PmxItemTable.Columns.NumberOfItemsPerPurchaseUnit.NAME]);
						 width  = string.IsNullOrEmpty(row[PmxItemTableQspac.Columns.Width.NAME].ToString()) ? width : Convert.ToDouble(row[PmxItemTableQspac.Columns.Width.NAME]);

						 value = width * numInBuy;
						 itemList.Add(itemCode, value);

						 itemCode = null;
						 width = 0;
						 numInBuy = 0;
						 value = 0;

					}
					ItemList.Set(itemList);
					barcodeController.QuantityCalculation(itemList);
					
				}	
			}
	
        }




	#endregion



	/// <summary>
	/// Gets the number of rows.
	/// </summary>
	/// <param name="ds">The ds.</param>
	/// <returns></returns>
	private int GetNumberOfRows(DataSet ds)
	{
		if (ds != null && ds.Tables.Count > 0)
		{
			return ds.Tables[0].Rows.Count;
		}
		return 0;
	}
	#region HELPERS

	#endregion


	#region BUILD QUERY

	/// <summary>
	/// Static class grouping our native queries
	/// </summary>
	internal static class BuildQuery
		{

			/// <summary>
			/// Gets the item info of the item groups
			/// </summary>
			/// <param name="itemGroups">The item groups</param>
			///  <param name="dbTool">The item groups</param>
			/// <returns></returns>
			public static string GetItemsInfo(List<string> itemGroups, DbTool dbTool)
			{

				string comma = "";
				DbQueryBuilder qb = dbTool.GetQueryBuilder();

				foreach (string group in itemGroups)
				{
					qb.
					Append(comma).
					Append("'").
					Append(group.Trim()).
					Append("'");
					comma = ", ";
				}

				string subSelect_for_groups = qb;


				qb.Clear();

				
				return qb.
					Select.
						EscTableColumnNameWithAlias(PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME, PmxItemTable.Columns.ItemCode.NAME).Comma.
						CRLF.
						EscTableColumnNameWithAlias(PmxItemTable.NAME, PmxItemTable.Columns.NumberOfItemsPerPurchaseUnit.NAME, PmxItemTable.Columns.NumberOfItemsPerPurchaseUnit.NAME).Comma.
						CRLF.
						EscTableColumnNameWithAlias(PmxItemTable.NAME, PmxItemTableQspac.Columns.Width.NAME, PmxItemTableQspac.Columns.Width.NAME).
						CRLF.
					From.
						EscTableName(PmxItemTable.NAME).
						CRLF.
					LeftJoin.
						EscTableName(SboItemGroupTable.NAME).On.
						EscTableColumnName(PmxItemTable.NAME, PmxItemTable.Columns.ItemGroupCode.NAME).Eq.
						EscTableColumnName(SboItemGroupTable.NAME, SboItemGroupTable.Columns.ItemsGroupCode.NAME).
						CRLF.
					Where.					
						EscTableColumnName(SboItemGroupTable.NAME, SboItemGroupTable.Columns.ItemsGroupName.NAME).
					   Append(" IN (").
					   Append(subSelect_for_groups).
					  Append(" ) ").
					  And.
					   EscTableColumnName(PmxItemTableQspac.NAME, PmxItemTableQspac.Columns.Width.NAME).IsNotNull.
					 ToString();

			}

		}

        #endregion
    }

}