﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Reflection;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;



// PmxWorkflowExecutionTypes.SUB_FLOW
namespace WorkflowScript_DisassemblyScriptQspac
{
    /// <summary>
    /// 
    /// </summary>
    public class WorkflowScript_DisassemblyScriptQspac : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        private static class Defs
        {
            /// <summary>
            /// The time to sleep in milliseconds
            /// </summary>
            public const int SleepTime = 2000;

			/// <summary>
			/// Hardcoded uom name
			/// </summary>
			public const string UOMNAME = "m";
		}

        // Input parameters
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<bool> UseItemQuantityCombinedScreen;

        ISboProviderService m_sboProviderService;

        // Output parameters
        // <none>

        // Sub-flows
        private WorkflowInstanceBase m_moveToProductionLineFlow;

        private string m_moveToProductionLineFlow_StrProductionOrderNumber;
        private string m_moveToProductionLineFlow_InputLocationCode;
        private string m_moveToProductionLineFlow_ProductionLineCode;
        private int? m_moveToProductionLineFlow_ProductionOrderDocEntry;
        private int? m_moveToProductionLineFlow_CurrentProdBatch;


        private WorkflowInstanceBase m_printItemLabelFlow;

        private bool m_printItemLabelFlow_BackRequested = false;
        private string m_printItemLabelFlow_LocationCode;
        private string m_printItemLabelFlow_NeededItemCode;
        private string m_printItemLabelFlow_NeededBatchNumber;
        private string m_printItemLabelFlow_NeededBatchNumber2;
        private DateTime? m_printItemLabelFlow_NeededBestBeforeDate;
        private int? m_printItemLabelFlow_NeededNrOfLabels;
        private bool m_printItemLabelFlow_AskBatchInfo;
        private double? m_printItemLabelFlow_EnteredQuantity;

        WorkflowInstanceBase m_enterQuantitySubFlow;

        protected bool m_subFlowBackRequested;
        protected Collection<string> m_subFlowListOfSerialNumbers;
        protected double m_subFlowQuantity;
        protected double? m_subFlowQuantityUom2;

		WorkflowInstanceBase m_quantityCalculationSubFlow;

		protected Dictionary<string,double> m_subFlowItemList;
		

		/// <summary>
		/// The m_enter batch attributes flow
		/// </summary>
		private WorkflowInstanceBase m_enterBatchAttributesFlow;
        /// <summary>
        /// The m_enter batch attributes flow_ list of batch attributes
        /// </summary>
        protected Collection<ItemTransactionalBatchAttributeInfo> m_enterBatchAttributesFlow_ListOfBatchAttributes;

        /// <summary>
        /// The m_enter quality values flow
        /// </summary>
        private WorkflowInstanceBase m_enterQualityValuesFlow;

        /// <summary>
        /// The m_enter quality values flow_ list of quality values
        /// </summary>
        protected Collection<QualityValueInfo> m_enterQualityValuesFlow_ListOfQualityValues;


        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_DisassemblyScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_DisassemblyScriptQspac(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
            m_moveToProductionLineFlow = this.Create("MoveToProductionLineScript");
            m_printItemLabelFlow = Create("PrintItemLabelScript");
            m_enterQuantitySubFlow = this.Create("EnterQuantityScript");
            m_enterBatchAttributesFlow = this.Create("EnterBatchAttributesScript");
            m_enterQualityValuesFlow = this.Create("EnterQualityValuesScript");
			m_quantityCalculationSubFlow = this.Create("QuantityCalculationQspac");
			
		}

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;


            //reset in case of error

            string deviceID = session.DeviceInstanceId;
            string query = null;

            Produmex.Foundation.Messages.Message msg = null;

            string productionLineCode = null;
            string productionLineCodeForDevice = null;
            PmxOseWarehouseInfo whsInfo = null;
            DataSet dsProductionLines = null;
            string inputLocationCode = null;
            string restLocationCode = null;
            string outputLocationCode = null;
            string initialErrorKey = null;
            DataSet dsStartedProductionOrder = null;

            int? productionOrderDocEntry = null;
            bool startedPRDFound = false;

            DataSet dsProductionOrder = null;
            string productionLineCodeFromProductionOrder = null;
            ProductionInfo productionInformation = null;
            DataSet dsCanBeLinedUpLocations = null;
            Collection<KeyValuePair<string, int>> listLinedUpLocations = null;
            bool lineUpLocationFromProductionTask = false;
            bool printItemLabel = false;
            string linedUpLocation = null;

            GeneratedBatchAndBestBeforeDateInfo batchAndBestBeforeDateInfo = null;
            string batchNumber = null;
            string batchNumber2 = null;
            DateTime? bestBeforeDate = null;

            DataSet dsIngredients = null;
            PmxItemInfo itemInfo = null;
            string[] allowedButtonKeysForProductionInfoScreen = null;
            string buttonKeySelectProductionTaskScreen = null;
            double? maximumQuantity = null;
            double? stillToQuantity = 0;
            DataSet dsItemsToReceive = null;
            string itemCodeToReceive = null;

            int numberOfReports = 0;
            string existingSSCC = null;
            int? existingLuid = null;
            string logisticCarrierCode = null;
            DataSet dsLogisticCarriers = null;
            string buttonKeyOtherTasksLogisticCarriers = null;
            DataSet ds = null;
            Collection<ItemTransactionalPackagingTypeInfo> currentListOfTransactionalPackagingTypeInfos = null;
            Collection<PackagingTypeInfo> listOfPackagingTypeInfosForScreen = null;
            PmxItemInfo itemInfoToReceive = null;
            int productionOrderLineToReceive = 0;
            double quantityToReceive = 0;
            double? quantityUom2ToReceive = null;
            Collection<string> listOfSerialNumbers = null;
            bool askReprint = false;
            bool needsToBeLinedUp = false;

            bool stopProduction = false;
            string buttonKeyOtherTasks = null;
            List<string> allowedButtonsForStopProductionTask = new List<string>();
            PmxUserGroupAuthorizationCodes authorizationStopButton = PmxUserGroupAuthorizationCodes.Enabled;//Default enabled
            PmxUserGroupAuthorizationCodes authorizationOnHoldButton = PmxUserGroupAuthorizationCodes.Enabled;//Default enabled
            string userCode = null;
            Collection<ConsumeItemInfo> listConsumeItemsToSend = new Collection<ConsumeItemInfo>();
            DataSet dsProductOnLine = null;
            Collection<KeyValuePair<string, ItemSetpointInfo>> listOfItemCodesAndQuantity = null;
            bool justStoppedPRD = false;
            Collection<ItemTransactionalBatchAttributeInfo> currentListOfTransactionalBatchAttributeInfos = null;

			DataSet dsSSCC = null;

            bool justEnteredItemQuantityCombined = false;

            //Get default settings
            bool useWaste = false;
            bool confirmProducedQtyAfterProduction = false;
            bool goBack = false;
            bool autoMoveAllItemsLinkedToBOM = false;
            bool allowToMoveFromRest = false;
            bool autoFillConsumedQtyFromPreparedQty = false;
            bool splitQtyInLogUnitsOfItemDefQtyOnLogUnit = false;
            bool skipConsumptionScreenForLinkedComponents = false;

            bool useItemQuantityCombinedScreen = UseItemQuantityCombinedScreen != null ? UseItemQuantityCombinedScreen.Get() : false;
            Collection<int> createdLUIDs = null;
            bool printOnGoBack = false;

            m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
              delegate(PmxDbConnection conn, object[] parameters)
              {

                  PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);
                  Dictionary<string, PmxParameterValue> plcParams = prdProv.GetParametersProductionController();


                  useWaste = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_USE_WASTE, false);
                  confirmProducedQtyAfterProduction = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_CONFIRM_QUANTITY, false);
                  autoMoveAllItemsLinkedToBOM = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_AUTO_MOVE_ITEMS_LINKED_TO_BOM, false);
                  allowToMoveFromRest = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_MOVE_TO_PROD_LINE_FROM_REST, false);
                  autoFillConsumedQtyFromPreparedQty = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_AUTO_FILL_CONSUMED_QTY_FROM_PREPARED_QTY, false);
                  splitQtyInLogUnitsOfItemDefQtyOnLogUnit = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_SPLIT_QTY_IN_LOG_UNITS_OF_ITEM_DEF_QTY_ON_LOG_UNIT, false);
                  skipConsumptionScreenForLinkedComponents = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_SKIP_CONSUMPTION_SCREEN_ON_FLOW_FOR_LINKED_COMPONENTS, false);

                  return null;
              });


            productionLineCodeForDevice = m_sboProviderService.InvokeMethodWithDbConnection<string>(false, false, null, null,
            delegate(PmxDbConnection conn, object[] parameters)
            {
                PmxOseProductionLineProvider plProv = new PmxOseProductionLineProvider(conn);
                return plProv.GetParentProductionLineForLocation(deviceID);
            });

            m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
               delegate(PmxDbConnection conn, object[] parameters)
               {
                   PmxUserProvider usProv = new PmxUserProvider();
                   userCode = conn.ConnectionStringObject.UserName;
                   Dictionary<string, PmxUserGroupAuthorizationCodes> authorizations = usProv.GetAllPmxGroupAuthorizationsOfUser(userCode, null, conn);//.TryGetValue(PmxUserAuthorizationColumnValues.ProductionFlowStopButton.CODE, out authorizationStopButton);
                   authorizations.TryGetValue(PmxUserAuthorizationColumnValues.ProductionFlowStopButton.CODE, out authorizationStopButton);
                   authorizations.TryGetValue(PmxUserAuthorizationColumnValues.ProductionFlowOnHoldButton.CODE, out authorizationOnHoldButton);

                   return null;
               });

			IBarcodeController barcodeController = null;
			m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
				delegate (PmxDbConnection conn, object[] parameters)
				{
					PmxBarcodeProvider bcProv = new PmxBarcodeProvider(conn);
					barcodeController = bcProv.GetBarcodeController();

					return null;
				});



			//bind parameters to sub flow
			m_moveToProductionLineFlow.BindParameters(
                this.DefaultCultureInfo,
                this.PmxOseCompany,
                new ReadWriteBinder<int?>("ProductionOrderDocEntry", this, "m_moveToProductionLineFlow_ProductionOrderDocEntry"),
                new ReadWriteBinder<string>("ProductionLineLocationCode", this, "m_moveToProductionLineFlow_ProductionLineCode"),
                new ReadWriteBinder<string>("InputLocationCode", this, "m_moveToProductionLineFlow_InputLocationCode"),
                 new ReadOnlyBinder<string>("ProductionOrderDocNum", this, "m_moveToProductionLineFlow_StrProductionOrderNumber"),
                 new ReadOnlyBinder<int?>("CurrentProdBatch", this, "m_moveToProductionLineFlow_CurrentProdBatch"));

            //initialize sub flows
            m_printItemLabelFlow.BindParameters(
                this.DefaultCultureInfo,
                this.PmxOseCompany,
                new ReadOnlyBinder<string>("LocationCode", this, "m_printItemLabelFlow_LocationCode"),
                new ReadOnlyBinder<string>("NeededItemCode", this, "m_printItemLabelFlow_NeededItemCode"),
                new ReadOnlyBinder<string>("NeededBatchNumber", this, "m_printItemLabelFlow_NeededBatchNumber"),
                new ReadOnlyBinder<string>("NeededBatchNumber2", this, "m_printItemLabelFlow_NeededBatchNumber2"),
                new ReadOnlyBinder<DateTime?>("NeededBestBeforeDate", this, "m_printItemLabelFlow_NeededBestBeforeDate"),
                new ReadOnlyBinder<int?>("NeededNrOfLabels", this, "m_printItemLabelFlow_NeededNrOfLabels"),
                new ReadOnlyBinder<bool>("AskBatchInfo", this, "m_printItemLabelFlow_AskBatchInfo"),
                new ReadWriteBinder<bool>("BackRequested", this, "m_printItemLabelFlow_BackRequested"),
                new ReadWriteBinder<double?>("EnteredQuantity", this, "m_printItemLabelFlow_EnteredQuantity")
                );


            whsInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), deviceID);


        Step_SelectProductionLine:

            #region SELECT PRODUCTION LINE


            query = BuildQuery.GetProductionLines(whsInfo.PmxWarehouseCode, deviceID);
            dsProductionLines = m_sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsProductionLines) > 0 &&
                !string.IsNullOrEmpty(productionLineCodeForDevice) &&
                !goBack)
            {
                productionLineCode = productionLineCodeForDevice;
                goto Step_GetProductionLineInfo;
            }

            if (this.GetNumberOfRows(dsProductionLines) == 1 && !goBack)
            {
                //auto select location
                productionLineCode = Convert.ToString(dsProductionLines.Tables[0].Rows[0][LocationDataSet.Columns.CODE.Name]);
                goto Step_GetProductionLineInfo;
            }

            goBack = false;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
               DefaultCultureInfo.Get(), BuildParamCollection(
               "InitialErrorKey", initialErrorKey,
               "TitleKey", "MSG_TITLE_SELECT_PRODUCTION_LINE",
               "LocationDS", dsProductionLines
               ));
            msg = WaitForMessage();
            initialErrorKey = null;
            if (msg.Name.EndsWith(".LocationSelected"))
            {
                productionLineCode = ExtractParameter<string>(msg.Parameters, "locationCode");
                goto Step_GetProductionLineInfo;
            }

            if (msg.Name.EndsWith(".RefreshRequested"))
            {
                goto Step_SelectProductionLine;
            }

            //back requested
            return;

            #endregion

        Step_GetProductionLineInfo:

            #region GET PRODUCTION LINE INFO


            if (this.GetNumberOfRows(dsProductionLines) > 0)
            {
                DataRow[] rows = dsProductionLines.Tables[0].Select(LocationDataSet.Columns.CODE.Name + " = '" + productionLineCode + "'");
                if (rows.Length > 0)
                {
                    DataRow row = rows[0];
                    inputLocationCode = row[PmxOseProductionLineTable.Columns.InputLocationCode.NAME] as string;
                    outputLocationCode = row[PmxOseProductionLineTable.Columns.OutputLocationCode.NAME] as string;
                    restLocationCode = row[PmxOseProductionLineTable.Columns.RestLocationCode.NAME] as string;
                    string warehouseCode = row[LocationDataSet.Columns.WAREHOUSECODE.Name] as string;
                    if (warehouseCode != whsInfo.PmxWarehouseCode)
                    {
                        whsInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), warehouseCode);
                    }

                }
                else
                {
                    //production line not found!
                    //This should not happen
                    goBack = true;
                    goto Step_SelectProductionLine;
                }
            }
            else
            {
                //production line not found!
                //This should not happen
                goBack = true;
                goto Step_SelectProductionLine;
            }

            goto Step_GetStartedProductionOrder;

            #endregion

        Step_GetStartedProductionOrder:

            #region GET STARTED PRODUCTION ORDER

            productionOrderDocEntry = null;
            startedPRDFound = false;
            justStoppedPRD = false;

            query = BuildQuery.GetStartedProductionOrders(productionLineCode);
            dsStartedProductionOrder = m_sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsStartedProductionOrder) == 0)
            {
                goto Step_SelectProductionOrder;
            }

            //Select first started prd
            startedPRDFound = true;

            if (Convert.ToString(dsStartedProductionOrder.Tables[0].Rows[0][ProductionOrderDataSet.Columns.PRODUCTION_ORDER_TYPE.Name]) != SboProductionOrderTypeValues.Disassembly.VALUE)
            {
                //Started production order is not a disassembly, so we cannot proceed.
                initialErrorKey = "MSG_ERROR_STARTED_PRODUCTION_ORDER_NO_DISASSEMBLY";
                //Force showing error
                goBack = true;
                goto Step_SelectProductionLine;
            }

            productionOrderDocEntry = Convert.ToInt32(dsStartedProductionOrder.Tables[0].Rows[0][ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name]);

            goto Step_SelectProductionOrder;

            #endregion

        Step_SelectProductionOrder:

            #region SELECT PRODUCTION ORDER

            query = BuildQuery.GetProductionOrders( productionLineCode, productionOrderDocEntry, m_sboProviderService.GetDbTool() );
            dsProductionOrder = m_sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsProductionOrder) == 1 &&
                productionOrderDocEntry.HasValue &&
                !goBack)
            {
                goto Step_GetProductionInfo;
            }

            // Show screen with list
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.ISelectProductionOrderScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_PACKAGING_ORDER",
                    "ProductionOrderDS", dsProductionOrder));
            msg = WaitForMessage();

            initialErrorKey = null;
            goBack = false;

            if (msg.Name.EndsWith(".BackRequested"))
            {
                goBack = true;
                goto Step_SelectProductionLine;
            }
            if (msg.Name.EndsWith(".RefreshRequested"))
            {
                goto Step_SelectProductionOrder;
            }
            if (!msg.Name.EndsWith(".ProductionOrderSelected"))
            {
                initialErrorKey = KnownMessageKeys.MSG_ERROR_OPERATION_NOT_SUPPORTED;
                goto Step_SelectProductionOrder;
            }

            //User selected a production order
            productionOrderDocEntry = ExtractParameter<int>(msg.Parameters, "productionOrderDocEntry");

            goto Step_GetProductionInfo;


            #endregion

        Step_GetProductionInfo:

            #region GET PRODUCTION INFO

            if (this.GetNumberOfRows(dsProductionOrder) > 0)
            {
                DataRow[] rows = dsProductionOrder.Tables[0].Select(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name + "=" + productionOrderDocEntry.Value.ToString());
                if (rows.Length > 0)
                {

                    productionLineCodeFromProductionOrder = rows[0][ProductionOrderDataSet.Columns.PRODUCTION_LINE_CODE.Name] as string;

                    productionInformation = new ProductionInfo();
                    productionInformation.Customer = rows[0][ProductionOrderDataSet.Columns.CUSTOMER_DESCRIPTION.Name] as string;
                    productionInformation.DueDate = Convert.ToDateTime(rows[0][ProductionOrderDataSet.Columns.DUE_DATE.Name]);
                    productionInformation.ItemNameToProduce = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.ITEMDESCRIPTION.Name]);
                    productionInformation.ItemCodeToProduce = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.ITEMCODE.Name]);
                    productionInformation.ProductionOrderNumber = Convert.ToInt32(rows[0][ProductionOrderDataSet.Columns.DOC_NUM.Name]);
                    productionInformation.ProductionComments = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.COMMENTS.Name]);
                    productionInformation.StillToProduce = Convert.ToDouble(rows[0][ProductionOrderDataSet.Columns.OPEN_QUANTITY.Name]);
                    productionInformation.NrOfBatches = Convertor.ConvertToInt32Nullable(rows[0][PmxProductionOrderTable.Columns.NrOfProdBatches.NAME]);
                    productionInformation.ProdBatchQuantity = Convertor.ConvertToDoubleNullable(rows[0][PmxProductionOrderTable.Columns.ProdBatchSize.NAME]);
                    productionInformation.PlannedQuantity = Convert.ToDouble(rows[0][PmxProductionOrderTable.Columns.PlannedQuantity.NAME]);
                    productionInformation.CurrentProdBatch = Convert.ToInt32(rows[0][PmxProductionOrderTable.Columns.CurrentProductionBatch.NAME]);

                    productionInformation.ProductionOrderDocEntry = productionOrderDocEntry.Value;

                    productionInformation.IsDisassembly = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.PRODUCTION_ORDER_TYPE.Name]) == SboProductionOrderTypeValues.Disassembly.VALUE;

                    if (productionInformation.NrOfBatches.HasValue && productionInformation.ProdBatchQuantity.HasValue)
                    {
                        productionInformation.StillToProduceCurrentProdBatch = productionInformation.ProdBatchQuantity.Value;
                    }

                    if (startedPRDFound)
                    {
                        goto Step_GetBatchBBD;
                    }
                    else
                    {
                        goto Step_ShowProductionInfo;
                    }
                    
                }
            }
            else
            {
                goto Step_SelectProductionLine;
            }

            #endregion

        Step_ShowProductionInfo:

            #region SHOW PRODUCTION INFO

            // Show screen with list
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.IShowProductionInfoScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_PRODUCTION_INFO",
                    "AllowedButtons", new string[1] { "MSG_BUTTON_START_PRODUCTION" },
                    "ProductionInfo", productionInformation
                    ));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".BackRequested"))
            {
                goBack = true;
                goto Step_SelectProductionOrder;
            }
            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                goto Step_StartProductionOrder;
            }



            #endregion

        Step_StartProductionOrder:

            #region START PRODUCTION ORDER

            if (!startedPRDFound)
            {

                try
                {

                    m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
                     delegate(PmxDbConnection conn, object[] parameters)
                     {

                         PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);
                         prdProv.StartProductionOrder(productionOrderDocEntry.Value);

                         return null;
                     });

                }
                catch (Exception ex)
                {
                    s_log.Error("", ex);
                    if (ex is ProdumexException)
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "PmxError", ((ProdumexException)ex).Error,
                                "ShowButton", true));
                        WaitForMessage();

                        if (((ProdumexException)ex).Error.ErrorCode == Produmex.Sbo.Logex.Data.ProdumexErrorCodes.ProductionOrderCannotBeStartedBeforeWeighingComponents)
                        {
                            goto Step_SelectProductionLine;
                        }
                    }
                    else
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
                                "NoTranslationOfMessageKey", true,
                                "IsError", true,
                                "ShowButton", true));
                        WaitForMessage();
                    }

                    //ask user if he wants to try again after error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                            "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                    msg = WaitForMessage();

                    if (msg.Name.EndsWith(".Yes"))
                    {
                        goto Step_StartProductionOrder;
                    }

                    goBack = true;
                    goto Step_SelectProductionLine;

                }
            }

            goto Step_AskStartQualityValues;

            #endregion

        Step_AskStartQualityValues:

            #region ASK START QUALITY VALUES

            m_enterQualityValuesFlow.BindParameters(
             this.DefaultCultureInfo,
             this.PmxOseCompany,
             new ReadOnlyBinder<PmxMomentOfCapture>("MomentOfCapture", PmxMomentOfCapture.Start),
             new ReadOnlyBinder<string>("BaseType", SboObjectTypes.ProductionOrder),
             new ReadOnlyBinder<int?>("BaseEntry", productionOrderDocEntry),
             new ReadOnlyBinder<bool>("OnlyAskWhenNotRegisteredYet", false),
             new ReadOnlyBinder<bool>("SaveData", true),

             new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
             new ReadWriteBinder<Collection<QualityValueInfo>>("ListOfQualityValues", this, "m_enterQualityValuesFlow_ListOfQualityValues")

             );

            m_subFlowBackRequested = false;
            this.StartSubFlowAndWait(m_enterQualityValuesFlow, true);

            if (m_subFlowBackRequested)
            {
                goBack = true;
                goto Step_SelectProductionLine;
            }

            goto Step_MoveStockToRest;

            #endregion

        Step_MoveStockToRest:

            #region MOVE STOCK TO REST

            try
            {
                m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
                 delegate(PmxDbConnection conn, object[] parameters)
                 {
                     PmxOseProductionLineProvider prov = new PmxOseProductionLineProvider(conn);
                     prov.MoveStockToRestLocation(productionLineCode, productionOrderDocEntry, productionInformation.CurrentProdBatch);

                     return null;
                 });

                if (autoMoveAllItemsLinkedToBOM && !justStoppedPRD &&
                    productionOrderDocEntry.HasValue)
                {

                    #region MOVE ALL ITEMS LINKED TO BOM

                    //get all itemcodes for production order
                    Collection<string> listOfItemCodesToMove = new Collection<string>();
                    listOfItemCodesToMove.Add(productionInformation.ItemCodeToProduce);
                    

                    //move all data linked to production order
                    m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
                         delegate(PmxDbConnection conn, object[] parameters)
                         {
                             PmxOseProductionLineProvider prov = new PmxOseProductionLineProvider(conn);
                             prov.MoveItemsFromInputToProductionLine(productionLineCode, listOfItemCodesToMove, allowToMoveFromRest, productionOrderDocEntry);
                             return null;
                         });


                    #endregion

                }

            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (ex is ProdumexException)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();
                }

                //ask user if he wants to try again after error
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                        "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_MoveStockToRest;
                }

                goBack = true;
                goto Step_SelectProductionLine;
            }

            if (buttonKeyOtherTasks == "MSG_BUTTON_NEXT_PRODUCTION_BATCH")
            {
                goto Step_SetProductionLineOnOrder;
            }
            else
            {
                if (justStoppedPRD)
                {
                    justStoppedPRD = false;
                    goto Step_GetStartedProductionOrder;
                }
                else
                {
                    goto Step_SetProductionLineOnOrder;
                }
            }
             
            #endregion

        Step_SetProductionLineOnOrder:

            #region SET PRODUCTION LINE ON ORDER

            if (string.IsNullOrEmpty(productionLineCodeFromProductionOrder))
            {
                try
                {
                    m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
                     delegate(PmxDbConnection conn, object[] parameters)
                     {
                         PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
                         prov.SetProductionLine(productionOrderDocEntry.Value, productionLineCode);

                         return null;
                     });

                }
                catch (Exception ex)
                {
                    s_log.Error("", ex);
                    if (ex is ProdumexException)
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "PmxError", ((ProdumexException)ex).Error,
                                "ShowButton", true));
                        WaitForMessage();
                    }
                    else
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
                                "NoTranslationOfMessageKey", true,
                                "IsError", true,
                                "ShowButton", true));
                        WaitForMessage();
                    }

                    //ask user if he wants to try again after error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                            "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                    msg = WaitForMessage();

                    if (msg.Name.EndsWith(".Yes"))
                    {
                        goto Step_SetProductionLineOnOrder;
                    }

                    goBack = true;
                    goto Step_SelectProductionLine;

                }
            }

            goto Step_GetBatchBBD;
           


            #endregion

        Step_GetBatchBBD:

            #region GET BATCH BBD

            batchAndBestBeforeDateInfo = m_sboProviderService.InvokeMethodWithDbConnection<GeneratedBatchAndBestBeforeDateInfo>(true, false, null, null,
             delegate(PmxDbConnection conn, object[] parameters)
             {
                 PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
                 return prov.GetBatchAndBestBeforeDate(productionOrderDocEntry.Value);
             });

            goto Step_EnterBatch;


            #endregion

        Step_EnterBatch:

            #region ENTER BATCH

            if (batchAndBestBeforeDateInfo != null &&
               batchAndBestBeforeDateInfo.HasBatchNumber)
            {
                batchNumber = batchAndBestBeforeDateInfo.BatchNumber;

                if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBatchNumber)
                {
                    //user can change the batch, so ask it
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                     DefaultCultureInfo.Get(), BuildParamCollection(
                         "InitialErrorKey", initialErrorKey,
                         "TitleKey", "MSG_TITLE_ENTER_BATCH_NUMBER_FOR_PRODUCTION",
                         "ForceDataEntry", true,
                         "ForceUppercase", true,
                         "AllowToGoBack", true,
                         "DefaultValue", batchNumber,
                         "Information", "MSG_DEFAULT_BATCH_NUMBER",
                         "MessageAddendum", batchAndBestBeforeDateInfo.BatchNumber
                         ));
                    msg = WaitForMessage();

                    // reset error
                    initialErrorKey = null;

                    if (msg.Name.EndsWith(".StringEntered"))
                    {
                        batchNumber = ExtractParameter<string>(msg.Parameters, "stringValue");
                    }
                    else if (msg.Name.EndsWith(".BackRequested"))
                    {
                        goBack = true;
                        goto Step_SelectProductionLine;
                    }
                }

            }

            goto Step_EnterBatch2;

            #endregion

        Step_EnterBatch2:

            #region ENTER BATCH2

            if (batchAndBestBeforeDateInfo.HasBatchNumber2)
            {
                batchNumber2 = batchAndBestBeforeDateInfo.BatchNumber2;

                if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBatchNumber2)
                {
                    //user can change the batch, so ask it
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                     DefaultCultureInfo.Get(), BuildParamCollection(
                         "InitialErrorKey", initialErrorKey,
                         "TitleKey", "MSG_TITLE_ENTER_BATCH_NUMBER2_FOR_PRODUCTION",
                         "ForceDataEntry", true,
                         "ForceUppercase", true,
                         "AllowToGoBack", true,
                         "DefaultValue", batchNumber2,
                         "Information", "MSG_DEFAULT_BATCH_NUMBER",
                         "MessageAddendum", batchAndBestBeforeDateInfo.BatchNumber2
                         ));
                    msg = WaitForMessage();

                    // reset error
                    initialErrorKey = null;

                    if (msg.Name.EndsWith(".StringEntered"))
                    {
                        batchNumber2 = ExtractParameter<string>(msg.Parameters, "stringValue");
                    }
                    else if (msg.Name.EndsWith(".BackRequested"))
                    {
                        if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBatchNumber)
                        {
                            goto Step_EnterBatch;
                        }

                        goBack = true;
                        goto Step_SelectProductionLine;
                    }
                }

            }

            goto Step_EnterBBD;

            #endregion

        Step_EnterBBD:

            #region ENTER BBD

            if (batchAndBestBeforeDateInfo.HasBestBeforeDate)
            {
                bestBeforeDate = batchAndBestBeforeDateInfo.BestBeforeDate;

				if (!batchAndBestBeforeDateInfo.GenerateBBDOnEveryReceipt)
				{
					if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBestBeforeDate)
					{
						//user can change the batch, so ask it
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterDateScreen),
						 DefaultCultureInfo.Get(), BuildParamCollection(
							 "InitialErrorKey", initialErrorKey,
							 "TitleKey", "MSG_TITLE_ENTER_BEST_BEFORE_DATE_FOR_PRODUCTION",
							 "ForceDataEntry", true,
							 "ForceUppercase", true,
							 "AllowToGoBack", true,
							 "DefaultDate", bestBeforeDate,
							 "Information", "MSG_DEFAULT_BEST_BEFORE_DATE"
							 ));
						msg = WaitForMessage();

						// reset error
						initialErrorKey = null;

						if (msg.Name.EndsWith(".DateEntered"))
						{
							bestBeforeDate = ExtractParameter<DateTime?>(msg.Parameters, "date");
						}
						else if (msg.Name.EndsWith(".BackRequested"))
						{
							if (batchAndBestBeforeDateInfo.HasBatchNumber2)
							{
								if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBatchNumber2)
								{
									goto Step_EnterBatch2;
								}
							}
							if (batchAndBestBeforeDateInfo.HasBatchNumber)
							{
								if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBatchNumber)
								{
									goto Step_EnterBatch;
								}
							}

							goBack = true;
							goto Step_SelectProductionLine;
						}
					}
				}
            }

            goto Step_ShowProductionTasks;

            #endregion

        Step_ShowProductionTasks:

            #region SHOW PRODUCTION TASKS

            lineUpLocationFromProductionTask = false;
            printItemLabel = false;
            printOnGoBack = false;

            query = BuildQuery.GetProductionOrders( productionLineCode, productionOrderDocEntry, m_sboProviderService.GetDbTool() );
            dsProductionOrder = m_sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsProductionOrder) > 0)
            {
                DataRow[] rows = dsProductionOrder.Tables[0].Select(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name + "=" + productionOrderDocEntry.Value.ToString());
                if (rows.Length > 0)
                {
                    productionInformation.Customer = rows[0][ProductionOrderDataSet.Columns.CUSTOMER_DESCRIPTION.Name] as string;
                    productionInformation.DueDate = Convert.ToDateTime(rows[0][ProductionOrderDataSet.Columns.DUE_DATE.Name]);
                    productionInformation.ItemNameToProduce = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.ITEMDESCRIPTION.Name]);
                    productionInformation.ItemCodeToProduce = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.ITEMCODE.Name]);
                    productionInformation.ProductionOrderNumber = Convert.ToInt32(rows[0][ProductionOrderDataSet.Columns.DOC_NUM.Name]);
                    productionInformation.ProductionComments = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.COMMENTS.Name]);
                    productionInformation.StillToProduce = Convert.ToDouble(rows[0][ProductionOrderDataSet.Columns.OPEN_QUANTITY.Name]);
                    productionInformation.NrOfBatches = Convertor.ConvertToInt32Nullable(rows[0][PmxProductionOrderTable.Columns.NrOfProdBatches.NAME]);
                    productionInformation.ProdBatchQuantity = Convertor.ConvertToDoubleNullable(rows[0][PmxProductionOrderTable.Columns.ProdBatchSize.NAME]);
                    productionInformation.PlannedQuantity = Convert.ToDouble(rows[0][PmxProductionOrderTable.Columns.PlannedQuantity.NAME]);
                    productionInformation.CurrentProdBatch = Convert.ToInt32(rows[0][PmxProductionOrderTable.Columns.CurrentProductionBatch.NAME]);

                    productionInformation.IsDisassembly = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.PRODUCTION_ORDER_TYPE.Name]) == SboProductionOrderTypeValues.Disassembly.VALUE;

                    if (productionInformation.NrOfBatches.HasValue && productionInformation.ProdBatchQuantity.HasValue)
                    {
                        productionInformation.StillToProduceCurrentProdBatch = productionInformation.ProdBatchQuantity.Value;
                    }


                  
                    if (productionInformation.NrOfBatches.HasValue && productionInformation.ProdBatchQuantity.HasValue && productionInformation.CurrentProdBatch.HasValue)
                    {
                        query = BuildQuery.GetQuantityAlreadyIssued( productionOrderDocEntry.Value, productionInformation.CurrentProdBatch.Value, m_sboProviderService.GetDbTool() );
                        DataSet tempDS = m_sboProviderService.RunView(false, null, null, query);
                        double alreadyProducedQty = Convert.ToDouble(tempDS.Tables[0].Rows[0][PmxProductionReceiptLineTable.Columns.Quantity.NAME]);
                        productionInformation.StillToProduceCurrentProdBatch = productionInformation.ProdBatchQuantity.Value - alreadyProducedQty;
                    }

                    itemInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), productionInformation.ItemCodeToProduce);


                    query = BuildQuery.GetIngredientsForProductionDisassembly( productionOrderDocEntry.Value, PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation, productionInformation.NrOfBatches.HasValue ? productionInformation.CurrentProdBatch : null, m_sboProviderService.GetDbTool() );
                    dsIngredients = m_sboProviderService.RunView(false, null, null, query);

                }
            }
            else
            {
                goto Step_SelectProductionLine;
            }

            allowedButtonKeysForProductionInfoScreen = new string[] { 
                        "MSG_BUTTON_DISASSEMBLE",
                        "MSG_BUTTON_CHANGE_LINED_UP_LOCATIONS",
                        "MSG_BUTTON_ADD_ITEMS_TO_USE", 
                        "MSG_BUTTON_PRINT_PRODUCT_LABEL",
                        "MSG_BUTTON_END_PRODUCTION_ORDER" };

            buttonKeySelectProductionTaskScreen = null;

            // Show screen with list
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.ISelectProductionTaskScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_TASK",
                    "AllowedButtons", allowedButtonKeysForProductionInfoScreen,
                    "DSPickItemsForProduction", dsIngredients,
                    "ProductionInfo", productionInformation
                    ));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".BackRequested"))
            {
                goBack = true;
                goto Step_SelectProductionOrder;
            }
            if (msg.Name.EndsWith(".RefreshRequested"))
            {
                goto Step_ShowProductionTasks;
            }
            if (msg.Name.EndsWith(".TimeRequested"))
            {
                goto Step_ShowProductionTasks;
                //goto Step_TimeRegistration;
            }
            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                buttonKeySelectProductionTaskScreen = ExtractParameter<string>(msg.Parameters, "buttonKey");

                switch (buttonKeySelectProductionTaskScreen)
                {
                    case "MSG_BUTTON_DISASSEMBLE":
                        goto Step_Produce;
                    case "MSG_BUTTON_CHANGE_LINED_UP_LOCATIONS":
                        lineUpLocationFromProductionTask = true;
                        goto Step_SelectItemToReceive;                    
                    case "MSG_BUTTON_ADD_ITEMS_TO_USE":

                        m_moveToProductionLineFlow_StrProductionOrderNumber = productionInformation.ProductionOrderNumber.ToString();
                        m_moveToProductionLineFlow_InputLocationCode = inputLocationCode;
                        m_moveToProductionLineFlow_ProductionLineCode = productionLineCode;
                        m_moveToProductionLineFlow_ProductionOrderDocEntry = productionOrderDocEntry;
                        m_moveToProductionLineFlow_CurrentProdBatch = productionInformation.CurrentProdBatch;

                        //call flow
                        base.StartSubFlowAndWait(m_moveToProductionLineFlow, true);


                        goto Step_ShowProductionTasks;

                    case "MSG_BUTTON_PRINT_PRODUCT_LABEL":

                        printItemLabel = true;

                        goto Step_SelectItemToReceive;

                    case "MSG_BUTTON_END_PRODUCTION_ORDER":
                        goto Step_StopProductionTask;
                    default:
                        initialErrorKey = "MSG_ERROR_OPERATION_NOT_SUPPORTED";
                        goto Step_ShowProductionTasks;

                }

            }



            #endregion

        Step_Produce:

            #region PRODUCE

            justEnteredItemQuantityCombined = false;

            maximumQuantity = m_sboProviderService.InvokeMethodWithDbConnection<double?>(true, false, null, null,
             delegate(PmxDbConnection conn, object[] parameters)
             {
                 PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
                 return prov.GetMaximumQuantityToProduce(productionOrderDocEntry.Value, null, PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation, whsInfo.StorageLocationCodeLogisticCarriers, itemInfo);
             });

            if (!maximumQuantity.HasValue ||
                maximumQuantity.Value <= 0)
            {
                initialErrorKey = "MSG_ERROR_NOT_ENOUGH_STOCK_TO_DISASSEMBLE";
                goto Step_ShowProductionTasks;
            }

            goto Step_SelectItemToReceive;

            #endregion

        Step_SelectItemToReceive:

            #region SELECT ITEM TO RECEIVE

            itemInfoToReceive = null;

            query = BuildQuery.GetItemsToReceive(productionOrderDocEntry.Value, lineUpLocationFromProductionTask, false, m_sboProviderService.GetDbTool());
            dsItemsToReceive = m_sboProviderService.RunView(false, null, null, query);

            if (lineUpLocationFromProductionTask &&
                this.GetNumberOfRows(dsItemsToReceive) == 0)
            {
                //no items to line up, so go back
                goto Step_ShowProductionTasks;
            }

            if (useItemQuantityCombinedScreen &&
                !lineUpLocationFromProductionTask)
            {
                //Skip this step, as we will ask it later on.
                goto Step_SelectLogisticCarrier;
            }

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.ISelectProductionOrderLineScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_PRODUCT",
                    "ProductionOrderLineDS", dsItemsToReceive
                    ));
            msg = WaitForMessage();

            //clear data
            initialErrorKey = null;

            if (msg.Name.EndsWith(".ProductionOrderLineSelected"))
            {
                int? tempProductionOrderLineToReceive = ExtractParameter<int?>(msg.Parameters, "productionOrderLineNum");

                if (tempProductionOrderLineToReceive.HasValue)
                {
                    productionOrderLineToReceive = tempProductionOrderLineToReceive.Value;

                    DataRow[] rows = dsItemsToReceive.Tables[0].Select(ProductionOrderLineDataSet.Columns.PRODUCTION_ORDER_LINE_NUM.Name + " = " + productionOrderLineToReceive.ToString());

                    if (rows.Length > 0)
                    {

                        itemCodeToReceive = rows[0][ProductionOrderLineDataSet.Columns.ITEMCODE.Name] as string;

                        itemInfoToReceive = m_sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), itemCodeToReceive);

                        needsToBeLinedUp = rows[0][ProductionOrderLineDataSet.Columns.HAS_TO_BE_LINED_UP.Name] as string == SboBooleanColumnValues.Y.VALUE;

                        if (lineUpLocationFromProductionTask)
                        {
                            goto Step_SelectLinedUpLocation;
                        }
                        else if (printItemLabel)
                        {
                            goto Step_PrintItemLabel;
                        }
                        else
                        {
                            goto Step_CheckMaximumQuantity;
                        }
                    }
                }
                
                //Go back
                goto Step_ShowProductionTasks;
                
            }

            //Go back
            goto Step_ShowProductionTasks;


            #endregion

        Step_PrintItemLabel:

            #region PRINT ITEM LABEL

            this.m_printItemLabelFlow_LocationCode = productionLineCode;
            if (itemInfoToReceive.HasBatchnumber)
            {
                this.m_printItemLabelFlow_NeededBatchNumber = batchNumber;
            }
            if (itemInfoToReceive.HasBestBeforeDate)
            {
                this.m_printItemLabelFlow_NeededBestBeforeDate = bestBeforeDate;
            }
            
            this.m_printItemLabelFlow_NeededItemCode = itemInfoToReceive.ItemCode;
            this.m_printItemLabelFlow_AskBatchInfo = false;

            this.m_printItemLabelFlow_EnteredQuantity = null;

            base.StartSubFlowAndWait(m_printItemLabelFlow, true);

            //call flow
            goto Step_ShowProductionTasks;

            #endregion

        Step_CheckMaximumQuantity:

            #region CHECK MAXIMUM QUANTITY

            //Check the maximum quantity that can be received for this item.

            maximumQuantity = m_sboProviderService.InvokeMethodWithDbConnection<double?>(true, false, null, null,
            delegate(PmxDbConnection conn, object[] parameters)
            {
                PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
                return prov.GetMaximumQuantityToReceiveForDisassembly(productionOrderDocEntry.Value, productionOrderLineToReceive, out stillToQuantity);
            });

            if (!maximumQuantity.HasValue ||
                maximumQuantity.Value == 0)
            {
                initialErrorKey = "MSG_ERROR_INSUFFICIENT_STOCK_TO_RECEIVE_ITEM_FROM_PRD";
                //goto Step_ShowProductionTasks;
                goto Step_Produce;
            }

            //clear some data
            existingSSCC = null;
            existingLuid = null;
            logisticCarrierCode = null;



            if (itemInfoToReceive.IsLogisticUnit)
            {
                goto Step_EnterPackagingQuantity;
            }
            else
            {
                goto Step_SelectLogisticCarrier;
            }


            #endregion

        Step_SelectLogisticCarrier:

            #region SELECT LOGISTIC CARRIERS

            bool foundLogisticCarriers = true;


            logisticCarrierCode = null;
            existingSSCC = null;
            existingLuid = null;

            if (itemInfoToReceive != null &&
                !string.IsNullOrEmpty(itemInfoToReceive.DefaultLogisticCarrierProduction))
            {
                logisticCarrierCode = itemInfoToReceive.DefaultLogisticCarrierProduction;
                goto Step_OtherTasksLogisticCarrier;
            }

            query = BuildQuery.GetLogisticCarriers(true, true, whsInfo.StorageLocationCodeLogisticCarriers, m_sboProviderService.GetDbTool());
            dsLogisticCarriers = m_sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsLogisticCarriers) > 0)
            {
                //We found logistic carriers, so we ask to select

                // Show screen with list
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectProductScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_SELECT_LOGISTIC_CARRIER",
                        "ProductDS", dsLogisticCarriers,
                        "ButtonKey", "MSG_BUTTON_OTHER_TASK"));
                msg = WaitForMessage();

                initialErrorKey = null;

                if (msg.Name.EndsWith(".BackRequested"))
                {
                    if (useItemQuantityCombinedScreen)
                    {
                        goto Step_ShowProductionTasks;
                    }
                    goto Step_Produce;
                }
                if (msg.Name.EndsWith(".RefreshRequested"))
                {
                    goto Step_SelectLogisticCarrier;
                }
                if (msg.Name.EndsWith(".ButtonClicked"))
                {
                    goto Step_OtherTasksLogisticCarrier;
                }
                if (!msg.Name.EndsWith(".ProductSelected"))
                {
                    initialErrorKey = KnownMessageKeys.MSG_ERROR_OPERATION_NOT_SUPPORTED;
                    goto Step_SelectLogisticCarrier;
                }

                logisticCarrierCode = ExtractParameter<string>(msg.Parameters, "itemCode");

                goto Step_EnterPackagingQuantity;


            }
            else
            {
                foundLogisticCarriers = false;

                goto Step_OtherTasksLogisticCarrier;
            }

            #endregion

        Step_OtherTasksLogisticCarrier:

            #region OTHER TASKS LOGISTIC CARRIERS

            buttonKeyOtherTasksLogisticCarriers = null;


            List<string> listOfButtons = new List<string>();

            if (!foundLogisticCarriers)
            {
                listOfButtons.Add("MSG_BUTTON_PRODUCE_NEW_SSCC");
            }
            else
            {
                listOfButtons.Add(string.IsNullOrEmpty(logisticCarrierCode) ? "MSG_BUTTON_NO_LOGISTIC_CARRIER" : "MSG_BUTTON_PROCEED");
            }

            listOfButtons.Add(productionInformation.IsDisassembly ? "MSG_BUTTON_DISASSEMBLE_ON_EXISTING_SSCC" : "MSG_BUTTON_PRODUCE_ON_EXISTING_SSCC");

            // Show screen with list
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_TASK",
                    "AllowedButtons", listOfButtons.ToArray()));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".BackRequested"))
            {
                if (this.GetNumberOfRows(dsLogisticCarriers) > 0)
                {
                    goto Step_SelectLogisticCarrier;
                }
                else
                {
                    goto Step_ShowProductionTasks;
                }
            }
            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                buttonKeyOtherTasksLogisticCarriers = ExtractParameter<string>(msg.Parameters, "buttonKey");

                if (buttonKeyOtherTasksLogisticCarriers == "MSG_BUTTON_NO_LOGISTIC_CARRIER" ||
                    buttonKeyOtherTasksLogisticCarriers == "MSG_BUTTON_PROCEED" ||
                    buttonKeyOtherTasksLogisticCarriers == "MSG_BUTTON_PRODUCE_NEW_SSCC")
                {
                    goto Step_EnterPackagingQuantity;
                }
                else
                {
                    goto Step_SelectExistingSSCC;
                }

            }

            #endregion

        Step_SelectExistingSSCC:

			#region SELECT EXISTING SSCC

			existingSSCC = null;
			existingLuid = null;

			query = BuildQuery.GetExistingSSSCCsLinkedToProductionOrder(productionOrderDocEntry.Value, m_sboProviderService.GetDbTool());

			dsSSCC = m_sboProviderService.RunView(false, null, null, query);

			// Show screen with list
			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.IIdentifyMultipleSSCCScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_IDENTIFY_SSCC",
					"BarcodeController", barcodeController,
					"SSCCDS", dsSSCC,
					"ForceDataEntry", true
				   ));
			msg = WaitForMessage();

			initialErrorKey = null;

			if (msg.Name.EndsWith(".BackRequested"))
			{
				goto Step_OtherTasksLogisticCarrier;
			}

			if (msg.Name.EndsWith(".SSCCScanned") || msg.Name.EndsWith(".ForwardRequested"))
			{
				existingSSCC = ExtractParameter<string>(msg.Parameters, "sscc");

				query = BuildQuery.GetProductOnSSCC(existingSSCC, null);
				ds = m_sboProviderService.RunView(false, null, null, query);

				if (this.GetNumberOfRows(ds) > 0)
				{
					//clear logistic carrier if it was set by default logistic carrier
					logisticCarrierCode = null;
					existingLuid = Convert.ToInt32((ds.Tables[0].Rows[0][PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME]));
					goto Step_EnterPackagingQuantity;
				}
				else
				{
					try
					{
						//new valid sscc
						m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
						 delegate(PmxDbConnection conn, object[] parameters)
						 {
							 PmxLogisticUnitIDProvider luidProv = new PmxLogisticUnitIDProvider((SboConnection)conn);
							 PmxSSCCProvider ssccProv = new PmxSSCCProvider((SboConnection)conn);
							 return existingLuid = luidProv.NewLogisticUnitID(existingSSCC, "");
						 });
						goto Step_EnterPackagingQuantity;
					}
					catch
					{
						initialErrorKey = "MSG_ERROR_INCORRECT_SSCC";
						goto Step_SelectExistingSSCC;
					}
				}
			}

			#endregion

        Step_EnterItemQuantityCombined:

            #region ENTER ITEM/QUANTITY COMBINED

            justEnteredItemQuantityCombined = false;
            printOnGoBack = false;

            //get all items to receive
            query = BuildQuery.GetItemsToReceive(productionOrderDocEntry.Value, lineUpLocationFromProductionTask, true, m_sboProviderService.GetDbTool());
            dsItemsToReceive = m_sboProviderService.RunView(false, null, null, query);

            //TODO: Add button to say pallet is full, and then go to print.

            PmxScaleDefinitionProvider.ScaleConfig scaleConfig = GetScaleConfig(session);

            //ask for quantity
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.IEnterItemQuantityCombinedScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_ENTER_QUANTITY_DISASSEMBLED",
                    "InitialQuantity", 0,
                    "NumberOfDigitsAfterDecimal", 6,
                    "MaximumQuantity", null,
                    "UOM", "",
                    "ProductionOrderLineDS", dsItemsToReceive,
                    "ButtonKey", existingLuid.HasValue ? "MSG_BUTTON_LOGISTIC_CARRIER_IS_FULL" : null,
                    "ScaleProviderType", scaleConfig.ProviderType,
                    "ScaleDecimals", scaleConfig.Decimals,
                    "ScaleMaxWeight", scaleConfig.MaxWeight,
                    "ScaleSettings", scaleConfig.Settings
                   ));
            msg = WaitForMessage();

            //clear data
            initialErrorKey = null;

            if (msg.Name.EndsWith(".ItemQuantityEntered"))
            {
                int? tempProductionOrderLineToReceive = ExtractParameter<int?>(msg.Parameters, "productionOrderLineNum");
                double enteredQuantity = ExtractParameter<double>(msg.Parameters, "quantity");

                if (tempProductionOrderLineToReceive.HasValue)
                {
                    productionOrderLineToReceive = tempProductionOrderLineToReceive.Value;

                    DataRow[] rows = dsItemsToReceive.Tables[0].Select(ProductionOrderLineDataSet.Columns.PRODUCTION_ORDER_LINE_NUM.Name + " = " + productionOrderLineToReceive.ToString());

                    if (rows.Length > 0)
                    {

                        itemCodeToReceive = rows[0][ProductionOrderLineDataSet.Columns.ITEMCODE.Name] as string;

                        itemInfoToReceive = m_sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), itemCodeToReceive);

                        needsToBeLinedUp = rows[0][ProductionOrderLineDataSet.Columns.HAS_TO_BE_LINED_UP.Name] as string == SboBooleanColumnValues.Y.VALUE;

                        //Check maximum quantity
                        maximumQuantity = m_sboProviderService.InvokeMethodWithDbConnection<double?>(true, false, null, null,
                          delegate(PmxDbConnection conn, object[] parameters)
                          {
                              PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
                              return prov.GetMaximumQuantityToReceiveForDisassembly(productionOrderDocEntry.Value, productionOrderLineToReceive, out stillToQuantity);
                          });

                        if (itemInfoToReceive.IsCatchWeightItem)
                        {
                            quantityUom2ToReceive = enteredQuantity;
                            quantityToReceive = PmxItemAllConnectionsProvider.CalculatePiecesFromWeight(enteredQuantity, itemInfoToReceive).Value;

							//Check if it is within tolerance
							if (!PmxItemAllConnectionsProvider.IsCatchWeightWithinTolerance(quantityToReceive, quantityUom2ToReceive.Value, itemInfoToReceive))
							{
								session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
															this.DefaultCultureInfo, BuildParamCollection(
															"MessageKey", "MSG_ERROR_CW_WEIGHT_NOT_WITHIN_TOLERANCE",
															"ShowButton", true,
															"IsError", true,
															"Parameters", new object[] { quantityUom2ToReceive.Value, quantityToReceive, itemInfoToReceive.CatchWeightWeightTolerancePercentage }));
								msg = WaitForMessage(); 
								
								goto Step_EnterItemQuantityCombined;
							}
                        }
                        else
                        {
                            quantityUom2ToReceive = null;
                            quantityToReceive = enteredQuantity;
                        }

                        if (!maximumQuantity.HasValue ||
                            maximumQuantity.Value == 0)
                        {
                            initialErrorKey = "MSG_ERROR_INSUFFICIENT_STOCK_TO_RECEIVE_ITEM_FROM_PRD";
                            goto Step_EnterItemQuantityCombined;
                        }
                        else if (maximumQuantity.Value > 0 &&
                            maximumQuantity.Value < quantityToReceive)
                        {
                            initialErrorKey = "MSG_ERROR_MAXIMUM_QUANTITY_EXCEEDED";
                            goto Step_EnterItemQuantityCombined;
                        }

                        


                    }
                }
                justEnteredItemQuantityCombined = true;
                goto Step_EnterBatchAttributes;

                //goto Step_SelectLinedUpLocation;
            }
            else if (msg.Name.EndsWith(".ButtonClicked"))
            {
                if (existingLuid.HasValue)
                {
                    createdLUIDs = new Collection<int>();
                    createdLUIDs.Add(existingLuid.Value);

                    goto Step_PrintProductionLabel;
                }
            }


            //go back
            if (existingLuid.HasValue)
            {
                //We have unfinished LUID, so print it out
                printOnGoBack = true;
                createdLUIDs = new Collection<int>();
                createdLUIDs.Add(existingLuid.Value);

                goto Step_PrintProductionLabel;
            }
            goto Step_ShowProductionTasks;



            #endregion

        Step_EnterPackagingQuantity:

            #region ENTER PACKAGING QUANTITY

            if (!maximumQuantity.HasValue ||
                maximumQuantity.Value == 0)
            {
                //no quantity allowed
                initialErrorKey = productionInformation.IsDisassembly ? "MSG_ERROR_NOT_ENOUGH_STOCK_TO_DISASSEMBLE" : "MSG_ERROR_NOT_ENOUGH_STOCK_TO_PRODUCE";
                goto Step_ShowProductionTasks;
            }

            if (useItemQuantityCombinedScreen)
            {
                //we ask for quantity on a different way
                goto Step_EnterBatchAttributes;
            }

            if ((itemInfoToReceive.HasBatchnumber || itemInfoToReceive.HasSecondBatchNumber || itemInfoToReceive.HasBestBeforeDate))
            {

                //batch not found, so we need to enter the quantity for the packaging types
                currentListOfTransactionalPackagingTypeInfos = SetItemTransactionalPackagingTypeInfoList(itemInfoToReceive);
                bool foundAtLeastOnePackagingTypeToShowOnScreen = false;
                bool isFirst = true;
                listOfPackagingTypeInfosForScreen = new Collection<PackagingTypeInfo>();
                foreach (PmxItemPackagingTypeInfo packType in itemInfoToReceive.ListOfPackagingTypes)
                {

                    PackagingTypeInfo info = new PackagingTypeInfo();

                    if (isFirst)
                    {
                        info.NumberOfDecimals = itemInfoToReceive.UomDecimals;
                    }

                    info.Quantity = packType.Quantity;
                    info.PackagingTypeCode = packType.PackagingTypeCode;
                    info.PackagingTypeName = packType.PackagingTypeName;

                    info.ShowOnScreen = packType.AskDuringReception;

                    listOfPackagingTypeInfosForScreen.Add(info);

                    if (!foundAtLeastOnePackagingTypeToShowOnScreen &&
                        info.ShowOnScreen)
                    {
                        foundAtLeastOnePackagingTypeToShowOnScreen = true;
                    }

                    isFirst = false;
                }

                if (foundAtLeastOnePackagingTypeToShowOnScreen)
                {
                    initialErrorKey = "MSG_ENTER_QUANTITY_OF_ALL_PACKAGING_TYPES";
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterPackagingQuantityScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_ENTER_QUANTITY_OF_ALL_PACKAGING_TYPES",
                        "ItemDescription", itemInfoToReceive.Description,
                        "ItemCode", itemInfoToReceive.ItemCode,
                        "ItemBarcode", itemInfoToReceive.CodeBars, "UOM", itemInfoToReceive.InventoryUom,
                        "ListOfPackagingTypeInfos", listOfPackagingTypeInfosForScreen,
                        "IsForEnteringDefaultPackagingQuantity", true,
                        "BatchNumber", batchNumber,
                        "BestBeforeDate", bestBeforeDate,
                        "SSCC", existingSSCC,
                        "DocNum", productionInformation.ProductionOrderNumber.ToString()
                        ));

                    msg = WaitForMessage();

                    // reset error
                    initialErrorKey = null;

                    if (msg.Name.EndsWith(".QuantitiesEntered"))
                    {
                        PackagingTypeInfos screenListOfPackagingTypeInfos = ExtractParameter<PackagingTypeInfos>(msg.Parameters, "listOfPackagingTypeInfos");

                        //build the list of packaging quantities
                        foreach (ItemTransactionalPackagingTypeInfo info in currentListOfTransactionalPackagingTypeInfos)
                        {
                            //try to get new qty from user
                            if (screenListOfPackagingTypeInfos.PackagingTypeInfoList.ContainsKey(info.PackagingTypeCode))
                            {
                                info.Quantity = screenListOfPackagingTypeInfos.PackagingTypeInfoList[info.PackagingTypeCode].Quantity;
                            }
                        }

                        goto Step_EnterBatchAttributes;
                    }
                    else if (msg.Name.EndsWith(".BackRequested"))
                    {
                        if (itemInfoToReceive.IsLogisticUnit)
                        {
                            goto Step_ShowProductionTasks;
                        }
                        goto Step_SelectLogisticCarrier;
                    }

                }

                goto Step_EnterBatchAttributes;
            }
            else
            {
                currentListOfTransactionalPackagingTypeInfos = new Collection<ItemTransactionalPackagingTypeInfo>();

                goto Step_EnterBatchAttributes;
            }

            #endregion

        Step_EnterBatchAttributes:

            #region ENTER BATCH ATTRIBUTES

            currentListOfTransactionalBatchAttributeInfos = null;

            if (itemInfoToReceive != null &&
                (itemInfoToReceive.HasBatchnumber || itemInfoToReceive.HasSecondBatchNumber || itemInfoToReceive.HasBestBeforeDate))
            {

                m_enterBatchAttributesFlow.BindParameters(
                          this.DefaultCultureInfo,
                          this.PmxOseCompany,
                          new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfoToReceive),
                          new ReadOnlyBinder<string>("BatchNumber", batchNumber),
                          new ReadOnlyBinder<string>("BatchNumber2", batchNumber2),
                          new ReadOnlyBinder<DateTime?>("BBD", bestBeforeDate),
                          new ReadOnlyBinder<bool>("SaveData", false),
						  new ReadOnlyBinder<BarcodeContainer>("Barcode", null),

                          new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
                          new ReadWriteBinder<Collection<ItemTransactionalBatchAttributeInfo>>("ListOfBatchAttributes", this, "m_enterBatchAttributesFlow_ListOfBatchAttributes")

                          );
                m_subFlowBackRequested = false;
                this.StartSubFlowAndWait(m_enterBatchAttributesFlow, true);

                if (m_subFlowBackRequested)
                {
                    if (itemInfoToReceive.IsLogisticUnit)
                    {
                        goto Step_ShowProductionTasks;
                    }
                    goto Step_SelectLogisticCarrier;
                }

                currentListOfTransactionalBatchAttributeInfos = m_enterBatchAttributesFlow_ListOfBatchAttributes;

                if (justEnteredItemQuantityCombined)
                {
                    //Used combined item/quantity screen, so now we need to check if batch attributes need to be entered
                    goto Step_SelectLinedUpLocation;
                }
                goto Step_EnterQuantity;
            }
            else
            {
                if (justEnteredItemQuantityCombined)
                {
                    //Used combined item/quantity screen, so now we need to check if batch attributes need to be entered
                    goto Step_SelectLinedUpLocation;
                }
                goto Step_EnterQuantity;
            }

            #endregion

        Step_EnterQuantity:

            #region ENTER QUANTITY

            quantityToReceive = 0;
            quantityUom2ToReceive = null;

            listOfSerialNumbers = null;
            m_subFlowListOfSerialNumbers = null;

            bool getNormalQuantity = true;
            lineUpLocationFromProductionTask = false;

            if (useItemQuantityCombinedScreen)
            {
                //we ask for quantity on a different way
                goto Step_EnterItemQuantityCombined;
            }

            if ((itemInfoToReceive.HasSerialNumber || itemInfoToReceive.HasSAPSerialNumber))
            {
                #region SERIAL NUMBER


                getNormalQuantity = false;

                //show screen to ask user what to do.
                List<string> buttonKeys = new List<string>(new Collection<string>() { "MSG_BUTTON_SERIAL_NUMBER", "MSG_BUTTON_SERIAL_NUMBER_RANGE" });

                if (!(//itemInfoToReceive.TrackLocationSerialNumber ||
                    (itemInfoToReceive.HasSAPSerialNumber && itemInfoToReceive.ManageMethod == SboItemManageMethod.OnEveryTransaction)))
                {
                    buttonKeys.Add("MSG_BUTTON_NO_SERIAL_NUMBER");
                }


                ToolBarButtons toolBarButtons = ToolBarButtons.Back;

                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_ENTER_QUANTITY_SERIAL_NUMBER_TASK",
                        "AllowedButtons", buttonKeys.ToArray(),
                        "EnabledToolBarButtons", toolBarButtons
                        ));
                msg = WaitForMessage();
                initialErrorKey = null;
                if (msg.Name.EndsWith(".ButtonClicked"))
                {
                    string buttonKeySerial = ExtractParameter<string>(msg.Parameters, "buttonKey");

                    if (buttonKeySerial == "MSG_BUTTON_SERIAL_NUMBER" ||
                        buttonKeySerial == "MSG_BUTTON_SERIAL_NUMBER_RANGE")
                    {
                        #region ENTER SERIAL NUMBERS


                        //get ITRI
                        int? itriKey = null;
                        if (itemInfoToReceive.HasBestBeforeDate ||
                            itemInfoToReceive.HasSecondBatchNumber ||
                            itemInfoToReceive.HasBatchnumber)
                        {
                            itriKey = m_sboProviderService.InvokeMethodWithDbConnection<int?>(false, false, null, null, new DelegateWithPmxDbConnection<int?>(ExecuteGetCreateItemTransactionalInfoKey), itemCodeToReceive, batchNumber, batchNumber2, bestBeforeDate);
                        }


                        bool isSerialNumberRange = (buttonKeySerial == "MSG_BUTTON_SERIAL_NUMBER_RANGE");

                        m_enterQuantitySubFlow.BindParameters(
                            this.DefaultCultureInfo,
                            this.PmxOseCompany,
                            new ReadOnlyBinder<string>("TitleKey", "MSG_TITLE_ENTER_QUANTITY"),
                            new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfoToReceive),
                            new ReadOnlyBinder<string>("BatchNumber", batchNumber),
                            new ReadOnlyBinder<DateTime?>("BestBeforeDate", bestBeforeDate),
                            new ReadOnlyBinder<int?>("ItriKey", itriKey),
                            new ReadOnlyBinder<string>("LocationCode", productionLineCode),
                            new ReadOnlyBinder<string>("SSCC", null),

                            new ReadOnlyBinder<string>("DocNum", productionInformation.ProductionOrderNumber.ToString()),
                            new ReadOnlyBinder<bool>("UsePickingInfo", false),

                            new ReadOnlyBinder<bool>("AllowZero", false),
                            new ReadOnlyBinder<double>("InitialQuantity", 1),
                            new ReadOnlyBinder<string>("MessageKeyStillQuantity", null),//"MSG_LBL_MAXIMUM_QUANTITY"),
                            new ReadOnlyBinder<double>("StillToQuantity", stillToQuantity.HasValue ? stillToQuantity.Value : 0),
                            new ReadOnlyBinder<double>("MaximumQuantity", maximumQuantity.Value),
                            new ReadOnlyBinder<double>("TotalQuantityToPick", 0),
                            new ReadOnlyBinder<string>("UOM", itemInfoToReceive.InventoryUom),
                            new ReadOnlyBinder<int>("NumberOfDigitsAfterDecimal", (itemInfoToReceive != null) ? itemInfoToReceive.UomDecimals : 0),

                            new ReadOnlyBinder<double>("InnerQuantity", 1),
                            new ReadOnlyBinder<string>("InnerUOM", ""),
                            new ReadOnlyBinder<bool>("ForceInputSerialNumbers", true),
                            new ReadOnlyBinder<string>("ObjectType", ((int)SboLinkDocumentType.ProductionOrder).ToString()),
                            new ReadOnlyBinder<int?>("DocEntry", productionOrderDocEntry.Value),
                            new ReadOnlyBinder<bool>("UseSerialNumberRange", isSerialNumberRange),
                            new ReadOnlyBinder<string>("LocationCode", productionLineCode),

                            new ReadOnlyBinder<int?>("LUID", null),


                            new ReadWriteBinder<double>("QuantityEntered", this, "m_subFlowQuantity"),
                            new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
                            new ReadWriteBinder<Collection<string>>("ListOfSerialNumbers", this, "m_subFlowListOfSerialNumbers")

                            );

                        m_subFlowBackRequested = false;
                        this.StartSubFlowAndWait(m_enterQuantitySubFlow, true);

                        if (m_subFlowBackRequested)
                        {
                            if (itemInfoToReceive.IsLogisticUnit)
                            {
                                goto Step_ShowProductionTasks;
                            }
                            goto Step_SelectLogisticCarrier;
                        }

                        listOfSerialNumbers = m_subFlowListOfSerialNumbers;

                        quantityToReceive = m_subFlowQuantity;

                        goto Step_SelectLinedUpLocation;

                        #endregion
                    }

                    getNormalQuantity = true;
                }

                if (msg.Name.EndsWith(".BackRequested"))
                {
                    if (itemInfoToReceive.IsLogisticUnit)
                    {
                        goto Step_ShowProductionTasks;
                    }
                    goto Step_SelectLogisticCarrier;
                }



                #endregion
            }

            if (getNormalQuantity)
            {


                if (itemInfoToReceive.ListOfPackagingTypes != null &&
                    currentListOfTransactionalPackagingTypeInfos.Count > 0 &&
                    !itemInfoToReceive.IsCatchWeightItem)
                {

                    //build list of packaging types
                    listOfPackagingTypeInfosForScreen = new Collection<PackagingTypeInfo>();

                    //add first one for the uom

                    PackagingTypeInfo info = new PackagingTypeInfo();

                    info.Quantity = 1;
                    info.PackagingTypeCode = null;
                    info.PackagingTypeName = itemInfoToReceive.InventoryUom == null ? itemInfoToReceive.ItemCode : itemInfoToReceive.InventoryUom;
                    info.QuantityPerPack = 1;
                    if (!string.IsNullOrEmpty(itemInfoToReceive.CodeBars))
                    {
                        info.Barcode.Add(itemInfoToReceive.CodeBars);
                    }
                    info.NumberOfDecimals = itemInfoToReceive.UomDecimals;
                    info.InitialQuantity = 1;
                    listOfPackagingTypeInfosForScreen.Add(info);


                    foreach (ItemTransactionalPackagingTypeInfo packType in currentListOfTransactionalPackagingTypeInfos)
                    {
                        info = new PackagingTypeInfo();

                        info.Quantity = packType.Quantity;
                        info.PackagingTypeCode = packType.PackagingTypeCode;
                        info.PackagingTypeName = GetPackTypeName(packType.PackagingTypeCode, itemInfoToReceive.ListOfPackagingTypes);
                        info.Barcode = GetPackBarcode(packType.PackagingTypeCode, itemInfoToReceive.ListOfPackagingTypes);

                        info.HideDuringEnteringQuantity = GetHideDuringEnteringQuantity(packType.PackagingTypeCode, itemInfoToReceive.ListOfPackagingTypes);

                        info.InitialQuantity = 0;

                        info.QuantityPerPack = packType.Quantity;

                        listOfPackagingTypeInfosForScreen.Add(info);
                    }


                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterPackagingQuantityScreen),
                                        DefaultCultureInfo.Get(), BuildParamCollection(
                                        "InitialErrorKey", initialErrorKey,
                                        "TitleKey", productionInformation.IsDisassembly ? "MSG_TITLE_ENTER_QUANTITY_DISASSEMBLED" : "MSG_TITLE_ENTER_QUANTITY_PRODUCED",
                                        "ItemDescription", itemInfoToReceive.Description,
                                        "ItemCode", itemInfoToReceive.ItemCode,
                                        "ItemBarcode", itemInfoToReceive.CodeBars,
                                        "UOM", itemInfoToReceive.InventoryUom,
                                        "ListOfPackagingTypeInfos", listOfPackagingTypeInfosForScreen,
                                        "IsForEnteringDefaultPackagingQuantity", false,
                                        "BatchNumber", batchNumber,
                                        "BestBeforeDate", bestBeforeDate,
                                        "SSCC", existingSSCC,
                                        "DocNum", productionInformation.ProductionOrderNumber.ToString(),
                                        "StillToQuantity", stillToQuantity.HasValue ? stillToQuantity.Value : 0,
                                        "MessageKeyStillQuantity", null, // "MSG_LBL_MAXIMUM_QUANTITY",
                                        "AllowZero", false,
                                        "MaximumQuantity", maximumQuantity.Value
                                        ));

                    msg = WaitForMessage();

                    // reset error
                    initialErrorKey = null;

                    if (msg.Name.EndsWith(".QuantitiesEntered"))
                    {
                        PackagingTypeInfos screenListOfPackagingTypeInfos = ExtractParameter<PackagingTypeInfos>(msg.Parameters, "listOfPackagingTypeInfos");

                        //build the list of packaging quantities
                        foreach (PackagingTypeInfo infoForQty in screenListOfPackagingTypeInfos.PackagingTypeInfoList.Values)
                        {
                            quantityToReceive += (infoForQty.Quantity * infoForQty.QuantityPerPack);
                        }

                        goto Step_SelectLinedUpLocation;
                    }
                    else if (msg.Name.EndsWith(".BackRequested"))
                    {
                        if (itemInfoToReceive.IsLogisticUnit)
                        {
                            goto Step_ShowProductionTasks;
                        }
                        goto Step_SelectLogisticCarrier;
                    }
                }
                else
                {

                    //Call enterquantity script
                    m_enterQuantitySubFlow.BindParameters(
                        this.DefaultCultureInfo,
                        this.PmxOseCompany,
                        new ReadOnlyBinder<string>("TitleKey", "MSG_TITLE_ENTER_QUANTITY_DISASSEMBLED"),
                        new ReadOnlyBinder<string>("TitleKeyWeight", "MSG_TITLE_ENTER_WEIGHT_DISASSEMBLED"),
                        new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfoToReceive),
                        new ReadOnlyBinder<string>("BatchNumber", batchNumber),
                        new ReadOnlyBinder<DateTime?>("BestBeforeDate", bestBeforeDate),
                        new ReadOnlyBinder<int?>("ItriKey", null),
                        new ReadOnlyBinder<string>("LocationCode", null),
                        new ReadOnlyBinder<string>("SSCC", null),

                        new ReadOnlyBinder<string>("DocNum", productionInformation.ProductionOrderNumber.ToString()),
                        new ReadOnlyBinder<bool>("UsePickingInfo", false),
                        new ReadOnlyBinder<bool>("IsAlternatePicking", false),

                        new ReadOnlyBinder<bool>("AllowZero", false),
                        new ReadOnlyBinder<double>("InitialQuantity", 0),
                        new ReadOnlyBinder<double>("StillToQuantity", stillToQuantity.HasValue ? stillToQuantity.Value : 0),
                        new ReadOnlyBinder<string>("MessageKeyStillQuantity", null ), //"MSG_LBL_MAXIMUM_QUANTITY"),
                        new ReadOnlyBinder<double>("MaximumQuantity", maximumQuantity.Value),
                        new ReadOnlyBinder<double>("TotalQuantityToPick", 0),
                        new ReadOnlyBinder<string>("UOM", itemInfoToReceive.InventoryUom),
                        new ReadOnlyBinder<int>("NumberOfDigitsAfterDecimal", (itemInfoToReceive != null) ? itemInfoToReceive.UomDecimals : 0),

                        new ReadOnlyBinder<double>("InnerQuantity", 1),
                        new ReadOnlyBinder<string>("InnerUOM", itemInfoToReceive.InventoryUom),
                        new ReadOnlyBinder<bool>("ForceInputSerialNumbers", false),
                        new ReadOnlyBinder<string>("ObjectType", Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.SboObjectTypes.ProductionOrder),
                        new ReadOnlyBinder<int?>("DocEntry", productionInformation.ProductionOrderDocEntry),
                        new ReadOnlyBinder<bool>("AutoFillQuantityForPackagingTypes", false),
                        new ReadOnlyBinder<bool>("OverrideAskingWeight", false),
                        new ReadWriteBinder<double?>("QuantityUom2Entered", this, "m_subFlowQuantityUom2"),
                        new ReadWriteBinder<double>("QuantityEntered", this, "m_subFlowQuantity"),
                        new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
                        new ReadWriteBinder<Collection<string>>("ListOfSerialNumbers", this, "m_subFlowListOfSerialNumbers")

                        );

                    m_subFlowBackRequested = false;
                    this.StartSubFlowAndWait(m_enterQuantitySubFlow, true);

                    if (m_subFlowBackRequested)
                    {
                        if (itemInfoToReceive.IsLogisticUnit)
                        {
                            goto Step_ShowProductionTasks;
                        }
                        goto Step_SelectLogisticCarrier;
                    }

                    listOfSerialNumbers = m_subFlowListOfSerialNumbers;
                    quantityToReceive = m_subFlowQuantity;
                    quantityUom2ToReceive = m_subFlowQuantityUom2;

                    goto Step_SelectLinedUpLocation;


                    //session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityWithItemInfoScreen),
                    //DefaultCultureInfo.Get(), BuildParamCollection(
                    //    "InitialErrorKey", initialErrorKey,
                    //    "TitleKey", productionInformation.IsDisassembly ? "MSG_TITLE_ENTER_QUANTITY_DISASSEMBLED" : "MSG_TITLE_ENTER_QUANTITY_PRODUCED",
                    //    "InitialQuantity", 1,
                    //    "NumberOfDigitsAfterDecimal", itemInfoToReceive.UomDecimals,
                    //    "MaximumQuantity", maximumQuantity.Value,
                    //    "UOM", itemInfoToReceive.InventoryUom,
                    //    "DocNum", productionInformation.ProductionOrderNumber.ToString(),
                    //    "ItemDescription", itemInfoToReceive.Description,
                    //    "ItemCode", itemCodeToReceive,
                    //    "ItemBarcode", itemInfoToReceive.CodeBars,
                    //    "BatchNumber", batchNumber,
                    //    "BestBeforeDate", bestBeforeDate,
                    //    "StillToQuantity", maximumQuantity.Value > 0 ? maximumQuantity.Value : 0,
                    //    "MessageKeyStillQuantity", "MSG_LBL_MAXIMUM_QUANTITY"
                    //    ));
                    //msg = WaitForMessage();

                    //// reset error
                    //initialErrorKey = null;

                    //if (msg.Name.EndsWith(".QuantityEntered"))
                    //{
                    //    quantityToReceive = ExtractParameter<double>(msg.Parameters, "quantity");
                    //    goto Step_ProduceStock;
                    //}
                    //else if (msg.Name.EndsWith(".BackRequested"))
                    //{
                    //    if (itemInfoToReceive.IsLogisticUnit)
                    //    {
                    //        goto Step_ShowProductionTasks;
                    //    }
                    //    goto Step_SelectLogisticCarrier;

                    //}

                }
            }




            #endregion

        Step_SelectLinedUpLocation:

            #region SELECT LINED UP LOCATION
            linedUpLocation = null;

            if (needsToBeLinedUp)
            {
                //Check if we find an already lined up location
                query = BuildQuery.GetCurrentLinedUpLocation(productionLineCode, productionOrderDocEntry.Value, productionOrderLineToReceive);
                ds = m_sboProviderService.RunView(false, null, null, query);

                if (this.GetNumberOfRows(ds) > 0 &&
                    !lineUpLocationFromProductionTask)
                {
                    //autoselect
                    linedUpLocation = ds.Tables[0].Rows[0][LocationDataSet.Columns.CODE.Name] as string;
                    goto Step_GenerateBBD;
                }


                //Get list of locations
                query = BuildQuery.GetCanBeLinedUpLocations(productionLineCode);
                dsCanBeLinedUpLocations = m_sboProviderService.RunView(false, null, null, query);

                listLinedUpLocations = new Collection<KeyValuePair<string, int>>();
                

                if (this.GetNumberOfRows(dsCanBeLinedUpLocations) == 0)
                {
                    initialErrorKey = "MSG_ERROR_NO_POSSIBLE_LINE_UP_LOCATION";
                    goBack = true;
                    goto Step_ShowProductionTasks;
                }




                //if (this.GetNumberOfRows(dsCanBeLinedUpLocations) == 1 &&
                //    !lineUpLocationFromProductionTask)
                //{
                //    //auto select first one
                //    linedUpLocation = dsCanBeLinedUpLocations.Tables[0].Rows[0][LocationDataSet.Columns.CODE.Name] as string;
                //}
                //else
                {

                    //Let user select it
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.ISelectLineUpLocationScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "InitialErrorKey", initialErrorKey,
                            "TitleKey", "MSG_TITLE_SELECT_LINE_UP_LOCATION",
                            "ProductionInfo", productionInformation,
                            "IngredientCode", itemCodeToReceive,
                            "IngredientDescription", itemInfoToReceive.Description,
                            "IngredientBarcode", itemInfoToReceive.CodeBars,
                            "DSLocations", dsCanBeLinedUpLocations
                            ));
                    msg = WaitForMessage();

                    initialErrorKey = null;

                    if (msg.Name.EndsWith(".BackRequested"))
                    {
                        goto Step_ShowProductionTasks;
                    }
                    if (msg.Name.EndsWith(".LocationSelected"))
                    {
                        linedUpLocation = ExtractParameter<string>(msg.Parameters, "locationCode");
                    }
                    else
                    {
                        //Refresh
                        goto Step_SelectLinedUpLocation;
                    }

                }

                if (!string.IsNullOrEmpty(linedUpLocation))
                {
                    listLinedUpLocations.Add(new KeyValuePair<string, int>(linedUpLocation, productionOrderLineToReceive));
                }


                goto Step_SetLinedUpLocationsToDB;
            }

            goto Step_GenerateBBD;

            #endregion

        Step_SetLinedUpLocationsToDB:

            #region SET LINED UP LOCATIONS TO DB


            try
            {

                if (listLinedUpLocations != null &&
                    listLinedUpLocations.Count > 0)
                {
                    m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
                       delegate(PmxDbConnection conn, object[] parameters)
                       {
                           PmxLinedUpLocationProvider prov = new PmxLinedUpLocationProvider(conn);
                           prov.AddLinedUpLocations(productionLineCode, productionOrderDocEntry.Value, listLinedUpLocations);

                           return null;
                       });

                }

                if (lineUpLocationFromProductionTask)
                {
                    goto Step_ShowProductionTasks;
                }
                else
                {
                    goto Step_GenerateBBD;
                }

            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (ex is ProdumexException)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();
                }

                //ask user if he wants to try again after error
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                        "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_SetLinedUpLocationsToDB;
                }

                goBack = true;
                goto Step_SelectProductionLine;

            }

			#endregion

		Step_GenerateBBD:
			#region GENERATE BBD

			if (batchAndBestBeforeDateInfo.GenerateBBDOnEveryReceipt)
			{
				if (batchAndBestBeforeDateInfo.HasBestBeforeDate)
				{
					bestBeforeDate = batchAndBestBeforeDateInfo.BestBeforeDate;

					if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBestBeforeDate)
					{
						//user can change the best before date, so ask it
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterDateScreen),
						 DefaultCultureInfo.Get(), BuildParamCollection(
							 "InitialErrorKey", initialErrorKey,
							 "TitleKey", "MSG_TITLE_ENTER_BEST_BEFORE_DATE_FOR_PRODUCTION",
							 "ForceDataEntry", true,
							 "ForceUppercase", true,
							 "AllowToGoBack", true,
							 "DefaultDate", bestBeforeDate,
							 "Information", "MSG_DEFAULT_BEST_BEFORE_DATE"
							 ));
						msg = WaitForMessage();

						// reset error
						initialErrorKey = null;

						if (msg.Name.EndsWith(".DateEntered"))
						{
							bestBeforeDate = ExtractParameter<DateTime?>(msg.Parameters, "date");
						}
						else if (msg.Name.EndsWith(".BackRequested"))
						{
							goto Step_ShowProductionTasks;
						}
					}

				}
			}

			goto Step_ProduceStock;
		#endregion

		Step_ProduceStock:

            #region PRODUCE STOCK

        Step_Produce_Again:


            //produce stock
            createdLUIDs = new Collection<int>();
            try
            {
                double splitQty = quantityToReceive;
                double? splitQtyUom2 = quantityUom2ToReceive;
                int splitInNrOfLogUnits = 1;
                if (splitQtyInLogUnitsOfItemDefQtyOnLogUnit && itemInfoToReceive.DefaultQuantityOnLogisticUnit > 0
                    && (!existingLuid.HasValue /*|| !existingLUIDIsInStock*/ ) /*&& advanceNoticeQty == 0*/ )
                {
                    splitInNrOfLogUnits = (int)SboQuantity.Ceil(splitQty / itemInfoToReceive.DefaultQuantityOnLogisticUnit, 0);
                }


                m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
                    delegate(PmxDbConnection conn, object[] parameters)
                    {
                        PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);

                        Collection<string> fullListOfSerialNumbers = listOfSerialNumbers;

                        Collection<ProductionReceiptLogisticUnitInfo> listOfLogUnits = new Collection<ProductionReceiptLogisticUnitInfo>();

                        for (int splitLogUnitIdx = 0; splitLogUnitIdx < splitInNrOfLogUnits; splitLogUnitIdx++)
                        {
                            ProductionReceiptLogisticUnitInfo logUnit = new ProductionReceiptLogisticUnitInfo();
                            logUnit.LUID = existingLuid;
                            logUnit.SetLUID = true;
                            logUnit.LogisticCarrierCode = logisticCarrierCode;

                            listOfLogUnits.Add(logUnit);

                            double logUnitQty = splitQty;
                            double? logUnitQtyUom2 = splitQtyUom2;

                            if (splitInNrOfLogUnits > 1)
                            {
                                logUnitQty = Math.Min(splitQty, itemInfoToReceive.DefaultQuantityOnLogisticUnit);
                                splitQty = Math.Round(splitQty - logUnitQty, itemInfoToReceive.UomDecimals);

                                if (itemInfoToReceive.IsCatchWeightItem)
                                {
                                    if (splitLogUnitIdx + 1 == splitInNrOfLogUnits)
                                    {
                                        //Keep remainder
                                        logUnitQtyUom2 = splitQtyUom2;
                                    }
                                    else
                                    {
                                        logUnitQtyUom2 = Math.Round((quantityUom2ToReceive.Value / splitInNrOfLogUnits) * logUnitQty, itemInfoToReceive.Uom2Decimals);
                                    }
                                    splitQtyUom2 = Math.Round(splitQtyUom2.Value - logUnitQtyUom2.Value, itemInfoToReceive.Uom2Decimals);
                                }
                            }

                            Collection<string> tempListOfSerialNumbers = null;

                            if (fullListOfSerialNumbers != null && fullListOfSerialNumbers.Count > 0)
                            {
                                tempListOfSerialNumbers = new Collection<string>();

                                for (int iLoop = 0; iLoop < logUnitQty; iLoop++)
                                {
                                    tempListOfSerialNumbers.Add(fullListOfSerialNumbers[iLoop]);
                                }

                            }

                            
                            ProductionReceiptItemInfo logUnitItem = new ProductionReceiptItemInfo();
                            logUnitItem.BatchInfo = batchAndBestBeforeDateInfo;
                            logUnitItem.BatchInfo.BatchNumber = batchNumber;
                            logUnitItem.BatchInfo.BatchNumber2 = batchNumber2;
                            logUnitItem.BatchInfo.BestBeforeDate = bestBeforeDate;
                            logUnitItem.ItemCode = itemCodeToReceive;
                            logUnitItem.ListOfPackagingTypes = currentListOfTransactionalPackagingTypeInfos;
                            logUnitItem.ListOfSerialNumbers = tempListOfSerialNumbers;
                            logUnitItem.Quantity = logUnitQty;
                            logUnitItem.QuantityUom2 = logUnitQtyUom2;
                            logUnitItem.ProductionOrderLineNum = productionOrderLineToReceive;

                            logUnitItem.ListOfBatchAttributes = currentListOfTransactionalBatchAttributeInfos;

                            logUnitItem.LinedUpLocation = linedUpLocation;


                            logUnit.ItemsOnLogisticUnit.Add(logUnitItem);


                            if (tempListOfSerialNumbers != null)
                            {
                                //remove serial numbers from full list, so next run will get new serial numbers
                                foreach (string serNum in tempListOfSerialNumbers)
                                {
                                    fullListOfSerialNumbers.Remove(serNum);
                                }
                            }

                        }

                        //produce         
                         createdLUIDs = prdProv.ReceiveDisassembly(productionOrderDocEntry.Value, listOfLogUnits, outputLocationCode, whsInfo.PmxWarehouseCode, productionInformation.CurrentProdBatch);



                        return null;

                    });

            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (ex is ProdumexException)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();
                }

                //ask user if he wants to try again after error
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                        "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_Produce_Again;
                }
                goto Step_ShowProductionTasks;

            }


            if (useItemQuantityCombinedScreen)
            {
                //Clear some data
                logisticCarrierCode = null;
                if (createdLUIDs.Count == 1)
                {
                    //Only keep LUID when there was only 1 created.
                    existingLuid = createdLUIDs[0];
                    createdLUIDs.Clear();

                    //Produced items
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                       DefaultCultureInfo.Get(), BuildParamCollection(
                           "InitialErrorKey", initialErrorKey,
                           "MessageKey", "MSG_ITEM_IS_DISASSEMBLED",
                           "ShowButton", true));

                    this.WaitForMessage();

                    goto Step_EnterItemQuantityCombined;
                }
                else
                {
                    //multiple LUID's created, so pint out the documents.
                    goto Step_PrintProductionLabel;
                }
            }

            goto Step_PrintProductionLabel;


            #endregion

        Step_PrintProductionLabel:

            #region PRINT PRODUCTION LABEL

            numberOfReports = 0;

            if (createdLUIDs.Count > 0)
            {
                try
                {
                    //trigger event
                    foreach (int createdLUID in createdLUIDs)
                    {
                        numberOfReports += m_sboProviderService.InvokeMethodWithDbConnection<int>(false, false, null, null,
                        delegate(PmxDbConnection conn, object[] parameters2)
                        {
                            PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
                            return printProv.PrintReportsForEvent(PmxPrintReportEventType.ProductionLogisticUnitProduced,
                                productionLineCode, deviceID, createdLUID, out askReprint);
                        });
                    }
                }
                catch (Produmex.Foundation.ProdumexException ex)
                {
                    s_log.Error(ex.ToString());
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "InitialErrorKey", initialErrorKey,
                            "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                            "MessageAddendum", ex.Message,
                            "PmxError", ex.Error,
                            "ShowButton", true));

                    msg = WaitForMessage();
                }
                catch (Exception ex)
                {
                    s_log.Error(ex.ToString());
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "InitialErrorKey", initialErrorKey,
                            "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                            "MessageAddendum", ex.Message,
                            "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                            "ShowButton", true));

                    msg = WaitForMessage();
                }


                if (numberOfReports > 0 && askReprint)
                {
                    goto Step_DocumentsPrinted;
                }

                if (!useItemQuantityCombinedScreen)
                {
                    //Only show message when we are not using combined item/quantity screen, because by using this, the item was already produced, and we came here when the user pressed 'Logistic unit full' button.
                    //Produced items
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                       DefaultCultureInfo.Get(), BuildParamCollection(
                           "InitialErrorKey", initialErrorKey,
                           "MessageKey", productionInformation.IsDisassembly ? "MSG_ITEM_IS_DISASSEMBLED" : "MSG_ITEM_IS_PRODUCED",
                           "ShowButton", true));

                    this.WaitForMessage();
                }


            }

            createdLUIDs.Clear();

            if (printOnGoBack)
            {
                goto Step_ShowProductionTasks;
            }

            //goto Step_ShowProductionTasks;
            goto Step_Produce;

            #endregion

        Step_DocumentsPrinted:

            #region DOCUMENTS PRINTED

            if (numberOfReports > 0 &&
                askReprint)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "MessageKey", "MSG_QUESTION_REPRINT_LABEL",
                        "TitleKey", "MSG_TITLE_REPRINT_LABEL"));

                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_PrintProductionLabel;
                }

            }

            createdLUIDs.Clear();

            if (printOnGoBack)
            {
                goto Step_ShowProductionTasks;
            }

            //goto Step_ShowProductionTasks;
            goto Step_Produce;


            #endregion

        Step_StopProductionTask:

            #region STOP PRODUCTION TASK

            stopProduction = false;
            buttonKeyOtherTasks = null;

            allowedButtonsForStopProductionTask = new List<string>();

            if (authorizationStopButton == PmxUserGroupAuthorizationCodes.Enabled)
            {
                allowedButtonsForStopProductionTask.Add("MSG_BUTTON_STOP_PRODUCTION");
            }
            if (authorizationOnHoldButton == PmxUserGroupAuthorizationCodes.Enabled)
            {
                allowedButtonsForStopProductionTask.Add("MSG_BUTTON_ON_HOLD_PRODUCTION");
            }
            allowedButtonsForStopProductionTask.Add("MSG_BUTTON_BACK_TO_PRODUCTION_LINE");
            if (productionInformation.NrOfBatches.HasValue && productionInformation.NrOfBatches.Value != productionInformation.CurrentProdBatch)
            {
                allowedButtonsForStopProductionTask.Add("MSG_BUTTON_NEXT_PRODUCTION_BATCH");
            }

            // Show screen with list
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_TASK",
                    "AllowedButtons", allowedButtonsForStopProductionTask.ToArray()
                    ));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_ShowProductionTasks;
            }
            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                buttonKeyOtherTasks = ExtractParameter<string>(msg.Parameters, "buttonKey");

                if (buttonKeyOtherTasks == "MSG_BUTTON_BACK_TO_PRODUCTION_LINE")
                {
                    goBack = true;
                    goto Step_SelectProductionLine;
                }
                else if (buttonKeyOtherTasks == "MSG_BUTTON_STOP_PRODUCTION")
                {
                    stopProduction = true;
                    goto Step_GetQuantityForConsumption;
                }
                else if (buttonKeyOtherTasks == "MSG_BUTTON_ON_HOLD_PRODUCTION")
                {
                    stopProduction = false;
                    goto Step_GetQuantityForConsumption;
                }
                else if (buttonKeyOtherTasks == "MSG_BUTTON_NEXT_PRODUCTION_BATCH")
                {

                    //ask user if he wants start the next batch
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "TitleKey", "MSG_TITLE_NEXT_PRODUCTION_BATCH",
                            "MessageKey", "MSG_QUESTION_NEXT_PRODUCTION_BATCH"));
                    msg = WaitForMessage();

                    if (msg.Name.EndsWith(".Yes"))
                    {
                        goto Step_GetQuantityForConsumption;
                    }

                    goto Step_ShowProductionTasks;

                }
                initialErrorKey = "MSG_ERROR_OPERATION_NOT_SUPPORTED";
                goto Step_StopProductionTask;
            }


            #endregion

        Step_GetQuantityForConsumption:

            #region GET QUANTITY FOR RUN

            string minQuantityColumnName = "MinQty";
            string maxQuantityColumnName = "MaxQty";

            query = BuildQuery.GetMinMaxForHeaderConsumption( productionOrderDocEntry, minQuantityColumnName, maxQuantityColumnName, m_sboProviderService.GetDbTool() );
            ds = m_sboProviderService.RunView(false, null, null, query);

            double minimumQtyHeader = 0;
            double maximumQtyHeader = double.MaxValue;
            double qtyToUseHeader = double.MaxValue;

            if (this.GetNumberOfRows(ds) > 0)
            {
                //We found some rows, to now get minimum and maximum
                foreach (DataRow row in ds.Tables[0].Rows)
                {
                    minimumQtyHeader = Math.Max(minimumQtyHeader, Convert.ToDouble(row[minQuantityColumnName]));
                    maximumQtyHeader = Math.Min(maximumQtyHeader, Convert.ToDouble(row[maxQuantityColumnName]));

                    qtyToUseHeader = Math.Min(qtyToUseHeader, Convert.ToDouble(row[PmxProductionReceiptLineTable.Columns.Quantity.NAME]));
                }

                //Round the quantity
                minimumQtyHeader = Math.Round(minimumQtyHeader, itemInfo.UomDecimals);
                maximumQtyHeader = Math.Round(maximumQtyHeader, itemInfo.UomDecimals);
                qtyToUseHeader = Produmex.Foundation.Utilities.FormattingHelper.CeilingForDecimals(qtyToUseHeader, itemInfo.UomDecimals);

                if (minimumQtyHeader > maximumQtyHeader)
                {
                    Collection<string> listOfItemCodes = new Collection<string>();
                    //Loop again to get items that are not received
                    foreach (DataRow row in ds.Tables[0].Rows)
                    {
                        double tempQty = Convert.ToDouble(row[maxQuantityColumnName]);

                        if (tempQty < minimumQtyHeader)
                        {
                            listOfItemCodes.Add(row[PmxProductionOrderLineTable.Columns.ItemCode.NAME] as string);
                        }
                    }

                    //Some items were not fully received.
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", "MSG_ERROR_NOT_ALL_ITEMS_RECEIVED_FOR_DISASSEMBLY",
                            "NoTranslationOfMessageKey", false,
                            "IsError", true,
                            "MessageAddendum", StringHelper.ConvertListToString(listOfItemCodes),
                            "ShowButton", true));
                    WaitForMessage();

                    goto Step_StopProductionTask;
                }
            }
           
            goto Step_StartConsume;
            

            #endregion

        Step_StartConsume:

			#region START CONSUME


			m_quantityCalculationSubFlow.BindParameters(		
			new ReadWriteBinder<Dictionary<string,double>>("ItemList", this, "m_subFlowItemList")		
			);
		
			this.StartSubFlowAndWait(m_quantityCalculationSubFlow, true);		


			query = BuildQuery.GetProductsOnLineDisassembly( productionOrderDocEntry.Value, productionLineCode, null, PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation ? whsInfo.StorageLocationCodeLogisticCarriers : null, m_sboProviderService.GetDbTool() );
            dsProductOnLine = m_sboProviderService.RunView(false, null, null, query);

            listConsumeItemsToSend = new Collection<ConsumeItemInfo>();


            listOfItemCodesAndQuantity = new Collection<KeyValuePair<string, ItemSetpointInfo>>();
            if (this.GetNumberOfRows(dsProductOnLine) > 0)
            {
                string currentItemCode = null;
                double qtyUsed = 0;
                double? qtyUom2Used = 0;
				bool itemGroupOfQtyCalc = false;
				double qtyCalculation = 1;
                Collection<string> listOfPreparedItemCodes = new Collection<string>();

                foreach (DataRow row in dsProductOnLine.Tables[0].Rows)
                {
					itemGroupOfQtyCalc = false;
					qtyCalculation = 1;

					double quantityOnLine = Convertor.ConvertToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITY.Name]);
                    double quantityUom2OnLine = Convertor.ConvertToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITY_UOM2.Name]);

                    double baseQuantity = Convertor.ConvertToDouble(row[ConsumeItemsForProductionDataSet.Columns.BASEQUANTITY.Name]);
                    double qtyTolPercentage = Convertor.ConvertToDouble(row[ConsumeItemsForProductionDataSet.Columns.QTYTOLPERCENTAGE.Name]);

                    double qtySetpoint = qtyToUseHeader;
                    double qtyToUse = double.NaN;
                    double? qtyUom2ToUse = null;


				
					if (currentItemCode != row[ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name] as string)
                    {
                        qtyUsed = 0;
                        qtyUom2Used = 0;
                        currentItemCode = row[ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name] as string;
                        //add the itemcode to the list because we will use it later to loop tru the IEnterConsumeItemsScreens

                        double qtyLowerLimit = Math.Round(minimumQtyHeader, itemInfo.UomDecimals);
                        double qtyUpperLimit = Math.Round(maximumQtyHeader, itemInfo.UomDecimals);
                        listOfItemCodesAndQuantity.Add(new KeyValuePair<string, ItemSetpointInfo>(currentItemCode, new ItemSetpointInfo(qtyLowerLimit, qtySetpoint, qtyUpperLimit)));

                    }

                    //set the used quantity;
                    if (double.IsNaN(qtyToUse))
                    {
                        qtyToUse = Math.Min(quantityOnLine, qtySetpoint - qtyUsed);
                        qtyUsed += qtyToUse;
                        if (itemInfo.IsCatchWeightItem)
                        {
                            if (quantityOnLine == qtyToUse)
                            {
                                qtyUom2ToUse = quantityUom2OnLine;
                            }
                            else
                            {
                                //Use default ratio
                                qtyUom2ToUse = PmxItemAllConnectionsProvider.CalculateWeightFromPieces(qtyToUse, itemInfo);
                            }
                            qtyUom2Used += qtyUom2ToUse;
                        }
                    }

                    row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name] = qtyToUse;
                    if (itemInfo.IsCatchWeightItem)
                    {
                        row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name] = qtyUom2ToUse;
                    }

                    //set the rest
                    row[ConsumeItemsForProductionDataSet.Columns.QUANTITYREST.Name] = quantityOnLine - qtyToUse;
                    if (itemInfo.IsCatchWeightItem)
                    {
                        row[ConsumeItemsForProductionDataSet.Columns.QUANTITYREST_UOM2.Name] = quantityUom2OnLine - qtyUom2ToUse;
                    }

                }


                listConsumeItemsToSend = new Collection<ConsumeItemInfo>();


                foreach (KeyValuePair<string, ItemSetpointInfo> itemCodeQuantity in listOfItemCodesAndQuantity)
                {
                    string currenItemCodeForConsume = itemCodeQuantity.Key;
                    ItemSetpointInfo totalToConsumeForItem = itemCodeQuantity.Value;

					PmxItemInfo itemInfoToConsume = m_sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), currenItemCodeForConsume);


					//QSpac recalc?
					if (m_subFlowItemList.ContainsKey(currenItemCodeForConsume))
					{
						m_subFlowItemList.TryGetValue(currenItemCodeForConsume, out qtyCalculation);
						itemGroupOfQtyCalc=true;

						//Header
						totalToConsumeForItem.LowerLimit = Math.Round(Convert.ToDouble(totalToConsumeForItem.LowerLimit) / Convert.ToDouble(qtyCalculation), itemInfoToConsume.UomDecimals);
						totalToConsumeForItem.Setpoint = Math.Round(Convert.ToDouble(totalToConsumeForItem.Setpoint) / Convert.ToDouble(qtyCalculation), itemInfoToConsume.UomDecimals);
						totalToConsumeForItem.UpperLimit = Math.Round(Convert.ToDouble(totalToConsumeForItem.UpperLimit) / Convert.ToDouble(qtyCalculation), itemInfoToConsume.UomDecimals);

					}
					else
					{
						itemGroupOfQtyCalc = false;
					}

					//copy structure
					DataSet dsConsumeItemsByItemCode = dsProductOnLine.Clone();

                    DataRow[] rows = dsProductOnLine.Tables[0].Select(ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name + " = '" + currenItemCodeForConsume + "'");

					
					foreach (DataRow row in rows)
                    {
						
						//recalc?
						if (itemGroupOfQtyCalc)
						{													
							
							row[ConsumeItemsForProductionDataSet.Columns.QUANTITYREST.Name] = Math.Round(Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYREST.Name]) / qtyCalculation, itemInfoToConsume.UomDecimals);
							row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name] = Math.Round(Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name]) / qtyCalculation, itemInfoToConsume.UomDecimals);
							row[ConsumeItemsForProductionDataSet.Columns.QUANTITY.Name] = Math.Round(Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITY.Name]) / qtyCalculation, itemInfoToConsume.UomDecimals);
							row[ConsumeItemsForProductionDataSet.Columns.BASEQUANTITY.Name] = Math.Round(Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.BASEQUANTITY.Name]) / qtyCalculation, itemInfoToConsume.UomDecimals);
							row[ConsumeItemsForProductionDataSet.Columns.UOM.Name] = Defs.UOMNAME;

							if (itemInfoToConsume.IsCatchWeightItem)
							{
								row[ConsumeItemsForProductionDataSet.Columns.QUANTITYREST_UOM2.Name] = Math.Round(Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYREST_UOM2.Name]) / qtyCalculation, itemInfoToConsume.UomDecimals);
								row[ConsumeItemsForProductionDataSet.Columns.QUANTITY_UOM2.Name] = Math.Round(Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITY_UOM2.Name]) / qtyCalculation, itemInfoToConsume.UomDecimals);
								row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name] = Math.Round(Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name]) / qtyCalculation, itemInfoToConsume.UomDecimals);
								row[ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE_UOM2.Name] = Math.Round(Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE_UOM2.Name]) / qtyCalculation, itemInfoToConsume.UomDecimals);

							}
						}
						
						//copy the items into the new ds
						dsConsumeItemsByItemCode.Tables[0].Rows.Add(row.ItemArray);
					}

                

                    DataSet dsConsumedItems = null;

                    if (skipConsumptionScreenForLinkedComponents && listOfPreparedItemCodes.Contains(currenItemCodeForConsume))
                    {
                        dsConsumedItems = dsConsumeItemsByItemCode;
                    }
                    else
                    {
                        //show screen
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.IEnterConsumeItemsScreen),
                           DefaultCultureInfo.Get(), BuildParamCollection(
                           "InitialErrorKey", initialErrorKey,
                           "TitleKey", "MSG_TITLE_ENTER_CONSUMED_QUANTITY",
                           "DSConsumeItems", dsConsumeItemsByItemCode,
                           "TotalQuantityToConsumeLowerLimit", totalToConsumeForItem.LowerLimit,
                           "TotalQuantityToConsume", totalToConsumeForItem.Setpoint,
                           "TotalQuantityToConsumeUpperLimit", totalToConsumeForItem.UpperLimit,
                           "NumbersOfDigitsAfterDecimal", itemInfoToConsume.UomDecimals,
                           "UseWaste", useWaste,
                           "IsCatchWeight", itemInfo.IsCatchWeightItem,
                           "NumbersOfDigitsAfterDecimalUom2", itemInfo.Uom2Decimals,
                           "TotalQuantityUom2ToConsume", PmxItemAllConnectionsProvider.CalculateWeightFromPieces(totalToConsumeForItem.Setpoint, itemInfo),
                           "WeightByPiece", itemInfo.DefaultWeightForPiece
                           ));
                        msg = WaitForMessage();
                        initialErrorKey = null;
                        if (msg.Name.EndsWith(".DataEntered"))
                        {
                            dsConsumedItems = ExtractParameter<DataSet>(msg.Parameters, "dsConsumedItems");
                        }
                        else
                        {
                            //go back
                            goto Step_StopProductionTask;
                        }

                    }


                    foreach (DataRow row in dsConsumedItems.Tables[0].Rows)
                    {
                        ConsumeItemInfo consume = new ConsumeItemInfo();

                        consume.BatchNumber1 = row[ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER.Name] as string;
                        consume.BatchNumber2 = row[ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER2.Name] as string;
                        if (row[ConsumeItemsForProductionDataSet.Columns.BESTBEFOREDATE.Name] is DateTime)
                        {
                            consume.BestBeforeDate = Convert.ToDateTime(row[ConsumeItemsForProductionDataSet.Columns.BESTBEFOREDATE.Name]);
                        }
                        consume.ItemCode = row[ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name] as string;
                        consume.ItemDescription = row[ConsumeItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name] as string;
                        consume.LocationCode = row[ConsumeItemsForProductionDataSet.Columns.LOCATIONCODE.Name] as string;
                        consume.LogisticUnitIdentKey = Convertor.ConvertToInt32Nullable(row[ConsumeItemsForProductionDataSet.Columns.LUID.Name]);
                        consume.QualityStatusCode = row[ConsumeItemsForProductionDataSet.Columns.QUALITYSTATUSCODE.Name] as string;
                        consume.QuantityUsed = Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name]);
                        consume.QuantityWaste = Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE.Name]);
                        consume.SSCC = row[ConsumeItemsForProductionDataSet.Columns.SSCC.Name] as string;

                        if (itemInfo.IsCatchWeightItem)
                        {
                            consume.QuantityUom2Used = Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name]);
                            consume.QuantityUom2Waste = Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE_UOM2.Name]);
                        }


						//QSpac recalc?
						if (m_subFlowItemList.ContainsKey(consume.ItemCode))
						{
							m_subFlowItemList.TryGetValue(consume.ItemCode, out qtyCalculation);
							itemGroupOfQtyCalc = true;
						}
						else
						{
							itemGroupOfQtyCalc = false;
						}
						if (itemGroupOfQtyCalc)
						{

							consume.QuantityUsed = Convert.ToDouble(Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name]) * qtyCalculation) ;
							consume.QuantityWaste = Convert.ToDouble(Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE.Name]) * qtyCalculation) ;

							if (itemInfo.IsCatchWeightItem)
							{
								consume.QuantityUom2Used = Convert.ToDouble(Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name]) * qtyCalculation);
								consume.QuantityUom2Waste = Convert.ToDouble(Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE_UOM2.Name]) * qtyCalculation) ;
							}

						}


						listConsumeItemsToSend.Add(consume);
                    }



                }

            }

            goto Step_AskEndQualityValues;

            #endregion

        Step_AskEndQualityValues:

            #region ASK END QUALITY VALUES

            m_enterQualityValuesFlow_ListOfQualityValues = null;

            m_enterQualityValuesFlow.BindParameters(
             this.DefaultCultureInfo,
             this.PmxOseCompany,
             new ReadOnlyBinder<PmxMomentOfCapture>("MomentOfCapture", PmxMomentOfCapture.End),
             new ReadOnlyBinder<string>("BaseType", SboObjectTypes.ProductionOrder),
             new ReadOnlyBinder<int?>("BaseEntry", productionOrderDocEntry),
             new ReadOnlyBinder<bool>("OnlyAskWhenNotRegisteredYet", false),
             new ReadOnlyBinder<bool>("SaveData", false),

             new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
             new ReadWriteBinder<Collection<QualityValueInfo>>("ListOfQualityValues", this, "m_enterQualityValuesFlow_ListOfQualityValues")

             );

            m_subFlowBackRequested = false;
            this.StartSubFlowAndWait(m_enterQualityValuesFlow, true);

            if (m_subFlowBackRequested)
            {
                goto Step_StopProductionTask;
            }

            goto Step_Consume;

            #endregion

        Step_Consume:

            #region CONSUME

            if (listConsumeItemsToSend != null &&
                listConsumeItemsToSend.Count > 0)
            {
                //consume stock
                try
                {
                    m_sboProviderService.InvokeMethodWithDbConnection<int?>(true, false, null, null,
                    delegate(PmxDbConnection conn, object[] parameters)
                    {

                        PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);
                        prdProv.Consume(productionOrderDocEntry.Value, listConsumeItemsToSend);

                        PmxQualityValueProvider qvProv = new PmxQualityValueProvider(conn);

                        if (m_enterQualityValuesFlow_ListOfQualityValues != null)
                        {
                            foreach (QualityValueInfo qvInfo in m_enterQualityValuesFlow_ListOfQualityValues)
                            {
                                using (PmxQualityValue qv = qvProv.GetNewBO())
                                {
                                    qv.BaseEntry = productionOrderDocEntry.Value;
                                    qv.BaseType =SboObjectTypes.ProductionOrder;
                                    qv.MomentOfCapture = qvInfo.MomentOfCapture;
                                    qv.QualityTypeCode = qvInfo.QualityTypeCode;
                                    qv.QualityValue = qvInfo.QualityValue;
                                    qv.ValueConvertor = qvInfo.ValueConvertor;

                                    qvProv.AddBO(qv);
                                }
                            }
                        }

                        return null;
                    });

                }
                catch (Exception ex)
                {
                    s_log.Error("", ex);
                    if (ex is ProdumexException)
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "PmxError", ((ProdumexException)ex).Error,
                                "ShowButton", true));
                        WaitForMessage();
                    }
                    else
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
                                "NoTranslationOfMessageKey", true,
                                "IsError", true,
                                "ShowButton", true));
                        WaitForMessage();
                    }

                    //ask user if he wants to try again after error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                            "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                    msg = WaitForMessage();

                    if (msg.Name.EndsWith(".Yes"))
                    {
                        goto Step_Consume;
                    }
                    goto Step_ShowProductionTasks;

                }
            }

            goto Step_UpdateProductionStatus;


            #endregion

        Step_UpdateProductionStatus:

            #region UPDATE PRODUCTION STATUS

            try
            {
                if (buttonKeyOtherTasks == "MSG_BUTTON_NEXT_PRODUCTION_BATCH")
                {

                    m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
                        delegate(PmxDbConnection conn, object[] parameters)
                        {
                            PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
                            prov.IncreaseCurrentProductionBatch(productionOrderDocEntry);
                            return null;
                        });

                    goto Step_MoveStockToRest;
                }
                else
                {

                    m_sboProviderService.InvokeMethodWithDbConnection<int?>(true, false, null, null,
                    delegate(PmxDbConnection conn, object[] parameters)
                    {

                        PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);

                        if (stopProduction)
                        {
                            prdProv.CloseOrder(productionOrderDocEntry.Value);
                        }
                        else
                        {
                            //on hold
                            prdProv.PutOnHold(productionOrderDocEntry.Value);
                        }

                        return null;
                    });
                }

            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (ex is ProdumexException)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();
                }

                //ask user if he wants to try again after error
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                        "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_UpdateProductionStatus;
                }
                goto Step_ShowProductionTasks;

            }

            //release lined up locations
            try
            {

                m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
                   delegate(PmxDbConnection conn, object[] parameters)
                   {

                       PmxLinedUpLocationProvider prov = new PmxLinedUpLocationProvider(conn);
                       prov.DeleteAllLinedUpLocationsForProductionLine(productionLineCode, productionOrderDocEntry.Value);

                       return null;
                   });

            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (ex is ProdumexException)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();
                }

            }

            justStoppedPRD = true;
            goto Step_MoveStockToRest;


            #endregion


            goto Step_SelectProductionLine;

   

        }

        #endregion

   

        #region HELPERS

        private static string GetPackTypeName(string packagingTypeCode, Collection<PmxItemPackagingTypeInfo> listOfPackagingTypes)
        {
            if (listOfPackagingTypes != null)
            {
                foreach (PmxItemPackagingTypeInfo packType in listOfPackagingTypes)
                {
                    if (packType.PackagingTypeCode == packagingTypeCode)
                    {
                        return packType.PackagingTypeName;
                    }
                }
            }

            return null;
        }

        private static Collection<string> GetPackBarcode(string packagingTypeCode, Collection<PmxItemPackagingTypeInfo> listOfPackagingTypes)
        {
            if (listOfPackagingTypes != null)
            {
                foreach (PmxItemPackagingTypeInfo packType in listOfPackagingTypes)
                {
                    if (packType.PackagingTypeCode == packagingTypeCode)
                    {
                        return packType.Barcodes;
                    }
                }
            }

            return null;
        }

        private static bool GetHideDuringEnteringQuantity(string packagingTypeCode, Collection<PmxItemPackagingTypeInfo> listOfPackagingTypes)
        {
            if (listOfPackagingTypes != null)
            {
                foreach (PmxItemPackagingTypeInfo packType in listOfPackagingTypes)
                {
                    if (packType.PackagingTypeCode == packagingTypeCode)
                    {
                        return packType.HideDuringEnteringQuantity;
                    }
                }
            }

            return true;
        }

        /// <summary>
        /// Executes the get create item transactional information key.
        /// </summary>
        /// <param name="conn">The connection.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static int? ExecuteGetCreateItemTransactionalInfoKey(PmxDbConnection conn, object[] parameters)
        {
            PmxItemTransactionalInfoProvider prov = new PmxItemTransactionalInfoProvider(conn);
            string itemCode = (string)parameters[0];
            string batchNumber = (string)parameters[1];
            string batchNumber2 = (string)parameters[2];
            DateTime? bbd = (DateTime?)parameters[3];

            return prov.GetCreateItemTransactionalInfoKey(itemCode, batchNumber, batchNumber2, bbd);
        }


        /// <summary>
        /// Executes the get PMX warehouse code.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseCode(PmxDbConnection conn, object[] parameters)
        {
            PmxOseProvider oseProv = new PmxOseProvider(conn);
            return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
        }

        /// <summary>
        /// Executes the get item info.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
        {
            PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
            return prov.GetCachedItemInfo((string)parameters[0]);
        }

        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null && ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }
            return 0;
        }


        private PmxScaleDefinitionProvider.ScaleConfig m_scaleConfig;

        private PmxScaleDefinitionProvider.ScaleConfig GetScaleConfig(Session session)
        {
            if ( m_scaleConfig == null )
            {
                m_sboProviderService.InvokeMethodWithDbConnection<object>( false, false, null, null,
                delegate( PmxDbConnection dbConn, object[] parameters )
                {
                    PmxScaleDefinitionProvider scaleDefProv = new PmxScaleDefinitionProvider();
                    m_scaleConfig = scaleDefProv.GetThinClientScales(session.DeviceInstanceId, dbConn);
                    
                    if (m_scaleConfig == null)
                    {
                        m_scaleConfig = new PmxScaleDefinitionProvider.ScaleConfig()
                        {
                            ProviderType = "-"  // No scale defined
                        };
                    }

                    return null;
                } );
            }
            return m_scaleConfig;
        }

        /// <summary>
        /// Sets the item transactional packaging type info list.
        /// </summary>
        /// <param name="itemInfo">The item info.</param>
        /// <returns></returns>
        private Collection<ItemTransactionalPackagingTypeInfo> SetItemTransactionalPackagingTypeInfoList(PmxItemInfo itemInfo)
        {
            //get the packaging types qty from item
            Collection<ItemTransactionalPackagingTypeInfo> listOfTransactionalPackagingTypeInfos = new Collection<ItemTransactionalPackagingTypeInfo>();

            foreach (PmxItemPackagingTypeInfo packInfo in itemInfo.ListOfPackagingTypes)
            {
                ItemTransactionalPackagingTypeInfo newPackInfo = new ItemTransactionalPackagingTypeInfo();

                newPackInfo.PackagingTypeCode = packInfo.PackagingTypeCode;
                newPackInfo.Quantity = packInfo.Quantity;
                newPackInfo.ItemCode = itemInfo.ItemCode;

                listOfTransactionalPackagingTypeInfos.Add(newPackInfo);

            }

            return listOfTransactionalPackagingTypeInfos;
        }


        #endregion

        #region CLASSES




        /// <summary>
        /// 
        /// </summary>
        public class ItemSetpointInfo
        {
            private double m_lowerLimit;
            private double m_upperLimit;
            private double m_setPoint;

            /// <summary>
            /// Gets or sets the lower limit.
            /// </summary>
            /// <value>The lower limit.</value>
            public double LowerLimit
            {
                get
                {
                    return m_lowerLimit;
                }
                set
                {
                    m_lowerLimit = value;
                }
            }

            /// <summary>
            /// Gets or sets the setpoint.
            /// </summary>
            /// <value>The setpoint.</value>
            public double Setpoint
            {
                get
                {
                    return m_setPoint;
                }
                set
                {
                    m_setPoint = value;
                }
            }

            /// <summary>
            /// Gets or sets the upper limit.
            /// </summary>
            /// <value>The upper limit.</value>
            public double UpperLimit
            {
                get
                {
                    return m_upperLimit;
                }
                set
                {
                    m_upperLimit = value;
                }
            }

            /// <summary>
            /// Initializes a new instance of the <see cref="ItemSetpointInfo"/> class.
            /// </summary>
            /// <param name="lowerLimit">The lower limit.</param>
            /// <param name="setpoint">The setpoint.</param>
            /// <param name="upperLimit">The upper limit.</param>
            public ItemSetpointInfo(double lowerLimit, double setpoint, double upperLimit)
            {
                LowerLimit = lowerLimit;
                Setpoint = setpoint;
                UpperLimit = upperLimit;
            }

            /// <summary>
            /// Initializes a new instance of the <see cref="ItemSetpointInfo"/> class.
            /// </summary>
            /// <param name="setpoint">The setpoint.</param>
            public ItemSetpointInfo(double setpoint)
            {
                LowerLimit = setpoint;
                Setpoint = setpoint;
                UpperLimit = setpoint;
            }
        }


        #endregion

        #region QUERY

        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        private class BuildQuery
        {


            /// <summary>
            /// Gets the current lined up location.
            /// </summary>
            /// <param name="String">The string.</param>
            /// <param name="productionLineCode">The production line code.</param>
            /// <param name="prdDocEntry">The PRD document entry.</param>
            /// <param name="prdLineNum">The PRD line number.</param>
            /// <returns></returns>
            public static string GetCurrentLinedUpLocation(string productionLineCode, int prdDocEntry, int prdLineNum)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT DISTINCT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);



                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseCanBeLinedUpLocationTable.NAME, PmxOseCanBeLinedUpLocationTable.Columns.CanBeLinedUpLocationCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);


                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLinedUpLocationTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinedUpLocationTable.NAME, PmxLinedUpLocationTable.Columns.LinedUpLocationCode.NAME);


                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinedUpLocationTable.NAME, PmxLinedUpLocationTable.Columns.ProductionLineCode.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionLineCode, CultureInfo.InvariantCulture));


                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinedUpLocationTable.NAME, PmxLinedUpLocationTable.Columns.ProductionOrderDocEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(prdDocEntry, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinedUpLocationTable.NAME, PmxLinedUpLocationTable.Columns.ProductionOrderLineNum.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(prdLineNum, CultureInfo.InvariantCulture));


                strBuilder.Append(" ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Name.NAME);


                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the can be lined up locations.
            /// </summary>
            /// <param name="productionLineCode">The production line code.</param>
            /// <returns></returns>
            public static string GetCanBeLinedUpLocations(string productionLineCode)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT DISTINCT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);



                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseCanBeLinedUpLocationTable.NAME, PmxOseCanBeLinedUpLocationTable.Columns.CanBeLinedUpLocationCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);


                strBuilder.Append(" WHERE ");

                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseCanBeLinedUpLocationTable.NAME, PmxOseCanBeLinedUpLocationTable.Columns.ParentCode.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionLineCode, CultureInfo.InvariantCulture));

               
                strBuilder.Append(" ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Name.NAME);


                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the minimum maximum for header consumption.
            /// </summary>
            /// <param name="productionOrderDocEntry">The production order document entry.</param>
            /// <param name="minQuantityColumnName">Minimum name of the quantity column.</param>
            /// <param name="maxQuantityColumnName">Maximum name of the quantity column.</param>
            /// <param name="dbTool">The db tool.</param>
            /// <returns></returns>
            public static string GetMinMaxForHeaderConsumption( int? productionOrderDocEntry, string minQuantityColumnName, string maxQuantityColumnName, DbTool dbTool )
            {
                /*select WOR1.LineNum, WOR1.ItemCode, RECEIPTS.Quantity, ISNULL(RECEIPTS.Quantity, 0) / WOR1.BaseQty AS headerQuantity
, (ISNULL(RECEIPTS.Quantity, 0) / WOR1.BaseQty) * 100.0 / (100.0 + WOR1.U_PMX_QTYTOLPCT) AS headerMinQuantity
, (ISNULL(RECEIPTS.Quantity, 0) / WOR1.BaseQty) * 100.0 / (100.0 - WOR1.U_PMX_QTYTOLPCT) AS headerMaxQuantity

from WOR1
inner join OWOR on OWOR.DocEntry = WOR1.DocEntry
left join
(
select IGN1.BaseEntry, IGN1.BaseLine, IGN1.BaseType, SUM(IGN1.Quantity) AS Quantity, oign.U_PMX_PRDR as productionRun
from IGN1
inner join OIGN on OIGN.DocEntry = ign1.DocEntry
GROUP BY IGN1.BaseEntry, IGN1.BaseLine, IGN1.BaseType, oign.U_PMX_PRDR
) RECEIPTS
on RECEIPTS.productionRun = ISNULL(owor.U_PMX_PRUN, 0)
AND RECEIPTS.BaseEntry = wor1.DocEntry
AND RECEIPTS.BaseLine = wor1.LineNum
AND RECEIPTS.BaseType = '202'

where wor1.DocEntry = 103
and U_PMX_OPTI = 'N'*/

                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

                string tableNameForReceipts = "RECEIPTS";

                strBuilder.Append("SELECT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);

                strBuilder.AppendFormat(", (" + strBuilder.IFNULL + "({0}, 0) / ({1} / {3})) AS {2} ",
                SqlCommandHelper.EscapeTableColumnName(tableNameForReceipts, PmxProductionReceiptLineTable.Columns.Quantity.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME),
					SqlCommandHelper.EscapeColumnName(PmxProductionReceiptLineTable.Columns.Quantity.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME)
                 );

				strBuilder.AppendFormat(", (" + strBuilder.IFNULL + "({0}, 0) / ({1} / {4})) * 100.0 / (100.0 + {2}) AS {3} ",
                 SqlCommandHelper.EscapeTableColumnName(tableNameForReceipts, PmxProductionReceiptLineTable.Columns.Quantity.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME),
                 SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityTolerancePercentage.NAME),
                 SqlCommandHelper.EscapeColumnName(minQuantityColumnName),
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME)
				  );

                strBuilder.AppendFormat(", (" + strBuilder.IFNULL + "({0}, 0) / ({1} / {4})) * 100.0 / (100.0 - {2}) AS {3} ",
                 SqlCommandHelper.EscapeTableColumnName(tableNameForReceipts, PmxProductionReceiptLineTable.Columns.Quantity.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME),
                 SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityTolerancePercentage.NAME),
                 SqlCommandHelper.EscapeColumnName(maxQuantityColumnName),
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME)
                  );



                strBuilder.Append("\r\nFROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);


                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);

                strBuilder.Append("\r\n LEFT JOIN (");
                {

                    strBuilder.Append("\r\n \r\n  SELECT ");

                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.BaseEntry.NAME);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.BaseLine.NAME);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.BaseType.NAME);

                    strBuilder.AppendFormat(", SUM({0}) AS {1} ",
                     SqlCommandHelper.EscapeTableColumnName(PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.Quantity.NAME),
                     SqlCommandHelper.EscapeColumnName(PmxProductionReceiptLineTable.Columns.Quantity.NAME)
                      );


                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionReceiptTable.NAME, PmxProductionReceiptTable.Columns.ProductionRun.NAME);
                  



                    strBuilder.Append("\r\nFROM ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionReceiptLineTable.NAME));

                    strBuilder.Append("\r\n INNER JOIN ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionReceiptTable.NAME));
                    strBuilder.Append(" ON ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionReceiptTable.NAME, PmxProductionReceiptTable.Columns.DocEntry.NAME);
                    strBuilder.Append(" = ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.DocEntry.NAME);

                    strBuilder.Append("\r\n GROUP BY ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.BaseEntry.NAME);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.BaseLine.NAME);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.BaseType.NAME);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionReceiptTable.NAME, PmxProductionReceiptTable.Columns.ProductionRun.NAME);
                  





                }
                strBuilder.Append("\r\n ) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(tableNameForReceipts));
                strBuilder.Append("\r\n ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, tableNameForReceipts, PmxProductionReceiptTable.Columns.ProductionRun.NAME);
                strBuilder.Append(" = ", strBuilder.IFNULL, "(");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionRun.NAME);
                strBuilder.Append(", 0) ");

                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, tableNameForReceipts, PmxProductionReceiptLineTable.Columns.BaseEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);

                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, tableNameForReceipts, PmxProductionReceiptLineTable.Columns.BaseLine.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME);



                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, tableNameForReceipts, PmxProductionReceiptLineTable.Columns.BaseType.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.ProductionOrder, CultureInfo.InvariantCulture));
               
                strBuilder.Append("\r\n WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));


                //Remove optional items
				strBuilder.Append("\r\n AND ", strBuilder.IFNULL, "(");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.Optional.NAME);
				strBuilder.AppendFormat(" , {0} ) = {0} ",
					SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append("\r\n AND ", strBuilder.IFNULL, "(");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UseForTimeRegistration.NAME);
                strBuilder.AppendFormat(" , {0} ) = {0} ",
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));


                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityTolerancePercentage.NAME);
                strBuilder.Append(" < ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(100, CultureInfo.InvariantCulture));

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the products on the line for disassembly.
            /// </summary>
            /// <param name="productionOrderDocEntry">The production order doc entry.</param>
            /// <param name="locationCode">The location code.</param>
            /// <param name="itemCode">The item code.</param>
            /// <param name="logisticCarrierLocationCode">The logistic carrier location code.</param>
            /// <returns></returns>
            public static string GetProductsOnLineDisassembly( int? productionOrderDocEntry, string locationCode, string itemCode, string logisticCarrierLocationCode, DbTool dbTool )
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

                strBuilder.Append( "SELECT " );
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER2.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, ConsumeItemsForProductionDataSet.Columns.BESTBEFOREDATE.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, ConsumeItemsForProductionDataSet.Columns.ISLOGISTICCARRIER.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, ConsumeItemsForProductionDataSet.Columns.LUID.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME, ConsumeItemsForProductionDataSet.Columns.QUALITYSTATUSCODE.Name);

                strBuilder.Append(", ", strBuilder.IFNULL, "( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
                strBuilder.Append(", 0) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.QUANTITY.Name));

                strBuilder.Append(", ", strBuilder.IFNULL, "( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QuantityUom2.NAME);
                strBuilder.Append(", 0) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.QUANTITY_UOM2.Name));


                strBuilder.AppendFormat(", " + strBuilder.IFNULL + "({0}, {1}) AS {2} ",
                  SqlCommandHelper.EscapeTableColumnName(SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME),
                  SqlCommandHelper.EscapeTableColumnName(SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME),
                  SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.UOM.Name));


                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UOM2.NAME, ConsumeItemsForProductionDataSet.Columns.UOM2.Name);


                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, ConsumeItemsForProductionDataSet.Columns.LOCATIONCODE.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.SSCC.NAME, ConsumeItemsForProductionDataSet.Columns.SSCC.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddDoubleQuantityColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.BASEQUANTITY.Name, 1);

                strBuilder.Append(", ");
                SqlCommandHelper.AddStringValueColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.OPTIONAL.Name, SboBooleanColumnValues.N.VALUE);

                strBuilder.Append(", ");
                SqlCommandHelper.AddDoubleQuantityColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QTYTOLPERCENTAGE.Name, 0);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.InternalKey.NAME, ConsumeItemsForProductionDataSet.Columns.INTERNALKEY.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYREST.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name);


                strBuilder.Append(", ");
                SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYREST_UOM2.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE_UOM2.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name);


                strBuilder.Append("\r\nFROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));

                strBuilder.Append("\r\nLEFT OUTER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));

                strBuilder.AppendFormat(" INNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxQualityStatusTable.NAME));
                strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.QualityStatusCode.NAME),
                    SqlCommandHelper.EscapeTableName(PmxQualityStatusTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxQualityStatusTable.Columns.Code.NAME));
                strBuilder.AppendFormat(" AND {0}.{1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxQualityStatusTable.NAME),
                    SqlCommandHelper.EscapeColumnName(PmxQualityStatusTable.Columns.CanBeUsedForProduction.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append("\r\nINNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME);


                strBuilder.Append("\r\n LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.InventoryUoMEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
                strBuilder.Append(" <> -1 ");

                strBuilder.Append("\r\nLEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);


                strBuilder.Append("\r\nWHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

                if (!string.IsNullOrEmpty(logisticCarrierLocationCode))
                {
                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

                }

                if (!string.IsNullOrEmpty(itemCode))
                {
                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

                }


                if (!string.IsNullOrEmpty(logisticCarrierLocationCode))
                {
                    #region UNION
                    strBuilder.Append("\r\nUNION ");

                    strBuilder.Append(" SELECT ");

                    strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER.Name));

                    strBuilder.Append(", ");
                    strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER2.Name));

                    strBuilder.Append(", ");
                    strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, ConsumeItemsForProductionDataSet.Columns.BESTBEFOREDATE.Name));

                    strBuilder.Append(", ");
                    strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name));

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, ConsumeItemsForProductionDataSet.Columns.ISLOGISTICCARRIER.Name);

                    strBuilder.Append(", ");
                    strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, ConsumeItemsForProductionDataSet.Columns.LUID.Name));

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME, ConsumeItemsForProductionDataSet.Columns.QUALITYSTATUSCODE.Name);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME, ConsumeItemsForProductionDataSet.Columns.QUANTITY.Name);

                    strBuilder.Append(", ", strBuilder.IFNULL, "( ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QuantityUom2.NAME);
                    strBuilder.Append(", 0) AS ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.QUANTITY_UOM2.Name));


                    strBuilder.AppendFormat(", " + strBuilder.IFNULL + "({0}, {1}) AS {2} ",
                      SqlCommandHelper.EscapeTableColumnName(SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME),
                      SqlCommandHelper.EscapeTableColumnName(SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME),
                      SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.UOM.Name));


                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UOM2.NAME, ConsumeItemsForProductionDataSet.Columns.UOM2.Name);


                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, ConsumeItemsForProductionDataSet.Columns.LOCATIONCODE.Name);

                    strBuilder.Append(", ");
                    strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.SSCC.NAME, ConsumeItemsForProductionDataSet.Columns.SSCC.Name));

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddDoubleQuantityColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.BASEQUANTITY.Name, 1);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddStringValueColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.OPTIONAL.Name, SboBooleanColumnValues.N.VALUE);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddDoubleQuantityColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QTYTOLPERCENTAGE.Name, 0);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.InternalKey.NAME, ConsumeItemsForProductionDataSet.Columns.INTERNALKEY.Name);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYREST.Name);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE.Name);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name);


                    strBuilder.Append(", ");
                    SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYREST_UOM2.Name);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE_UOM2.Name);

                    strBuilder.Append(", ");
                    SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name);

                    strBuilder.Append("\r\nFROM ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));

                    strBuilder.Append("\r\nLEFT OUTER JOIN ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(" ON ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
                    strBuilder.Append(" = ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);

                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(logisticCarrierLocationCode, CultureInfo.InvariantCulture));

                    strBuilder.Append("\r\nINNER JOIN ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                    strBuilder.Append(" ON ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
                    strBuilder.Append(" = ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME);

                    strBuilder.Append("\r\n LEFT JOIN ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
                    strBuilder.Append(" ON ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
                    strBuilder.Append(" = ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.InventoryUoMEntry.NAME);
                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
                    strBuilder.Append(" <> -1 ");

                    strBuilder.Append("\r\nLEFT JOIN ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                    strBuilder.Append(" ON ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
                    strBuilder.Append(" = ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);



                    strBuilder.Append("\r\nWHERE ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                    if (!string.IsNullOrEmpty(itemCode))
                    {
                        strBuilder.Append(" AND ");
                        SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
                        strBuilder.Append(" = ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

                    }

                    #endregion
                }


                strBuilder.Append("\r\nORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.OPTIONAL.Name));
                strBuilder.Append(" DESC, ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name));

                return strBuilder.ToString();
            }


            /// <summary>
            /// Command for getting all products order by name
            /// </summary>
            /// <param name="sscc">The SSCC.</param>
            /// <param name="locationCode">The location code.</param>
            /// <returns>
            /// Command for getting all products order by name
            /// </returns>
            public static string GetProductOnSSCC(string sscc, string locationCode)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.NAME, PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME, PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME));


                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.QualityStatusCode.NAME));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME);

                strBuilder.Append(", SUM( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
                strBuilder.Append(" ) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.Quantity.NAME));


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
                strBuilder.Append(" ON ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.Columns.ItemCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.Columns.ItemCode.NAME));

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);


                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));

                if (!string.IsNullOrEmpty(locationCode))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
                }


                strBuilder.Append(" GROUP BY ");

                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemName.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);


                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));
                return strBuilder.ToString();
            }

            /// <summary>
            /// Command for getting all logistic carriers order by name
            /// </summary>
            /// <param name="onlyInStock">if set to <c>true</c> the system will get the logistic carriers that are in stock and have no products on it (LUID IS NULL)</param>
            /// <param name="filterOutFrozenItems">if set to <c>true</c> [filter out frozen items].</param>
            /// <returns>
            /// Command for getting all logistic carriers order by name
            /// </returns>
            public static string GetLogisticCarriers(bool onlyInStock, bool filterOutFrozenItems, string storLocCode, DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
                strBuilder.Append("SELECT DISTINCT ");

                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME, ProductDataSet.Columns.PRODUCT_CODE.Name );

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, SboItemTable.NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name );

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, SboItemTable.NAME, SboItemTable.Columns.CodeBars.NAME, ProductDataSet.Columns.GTIN.Name );

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticCarrier.NAME));

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));

                if (onlyInStock)
                {
                    strBuilder.Append(" INNER JOIN ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(" ON ", strBuilder.AsIdentifier(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME));
                    strBuilder.Append(" = ", strBuilder.AsIdentifier(SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME));
                    strBuilder.Append(" AND ", strBuilder.AsIdentifier(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));
                    strBuilder.Append(" IS NULL ");
                }

                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.InventoryItem.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticCarrier.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                if (!string.IsNullOrEmpty(storLocCode))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(storLocCode, CultureInfo.InvariantCulture));
                }

                if (filterOutFrozenItems)
                {
                    Produmex.Sbo.Logex.Data.SboCommands.GetFrozenItemFilterSQLWherePart(strBuilder, " AND ");
                }


                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CustomItemDescr.NAME));
                return strBuilder.ToString();

            }


            /// <summary>
            /// Gets the items to receive.
            /// </summary>
            /// <param name="productionOrderDocEntry">The production order document entry.</param>
            /// <param name="onlyHasToBeLinedUp">if set to <c>true</c> [only has to be lined up].</param>
            /// <param name="useItemNameAsDescription">if set to <c>true</c> [use item name as description].</param>
            /// <param name="dbTool">The database tool.</param>
            /// <returns></returns>
            public static string GetItemsToReceive(int productionOrderDocEntry, bool onlyHasToBeLinedUp, bool useItemNameAsDescription, DbTool dbTool)
            {

                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

                strBuilder.Append("SELECT DISTINCT ");
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME, ProductionOrderLineDataSet.Columns.ITEMCODE.Name );

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, SboItemTable.NAME, useItemNameAsDescription? SboItemTable.Columns.ItemName.NAME : PmxItemTable.Columns.CustomItemDescr.NAME, ProductionOrderLineDataSet.Columns.ITEMDESCRIPTION.Name );

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, SboItemTable.NAME, SboItemTable.Columns.CodeBars.NAME, ProductDataSet.Columns.GTIN.Name );

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.Optional.NAME, ProductionOrderLineDataSet.Columns.IS_OPTIONAL.Name );

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME, ProductionOrderLineDataSet.Columns.PRODUCTION_ORDER_LINE_NUM.Name );


                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME, ProductionOrderLineDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name );

                //Only needed for screen
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME, ProductionOrderLineDataSet.Columns.QUANTITY.Name );

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME, ProductionOrderLineDataSet.Columns.HAS_TO_BE_LINED_UP.Name );

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.InventoryUom.NAME, ProductionOrderLineDataSet.Columns.UOM.Name);
				
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UOM2.NAME, ProductionOrderLineDataSet.Columns.UOM2.Name);


                strBuilder.Append("\r\n FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);



                strBuilder.Append("\r\n WHERE ");

                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));
                
                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.InventoryItem.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append("\r\n AND ", strBuilder.IFNULL, "(");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UseForTimeRegistration.NAME);
                strBuilder.AppendFormat(", {0} ) = {0} ",
                SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

                if (onlyHasToBeLinedUp)
                {

                    strBuilder.Append("\r\n AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME);
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
                }


                strBuilder.Append("\r\n ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.Optional.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME);
                

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the ingredients for production for disassembly.
            /// </summary>
            /// <param name="productionOrderDocEntry">The production order doc entry.</param>
            /// <param name="areLogisticCarriersStoredOnOneLocation">if set to <c>true</c> [are logistic carriers stored on one location].</param>
            /// <param name="currentProdBatchNumber">The current product batch number.</param>
            /// <returns></returns>
            public static string GetIngredientsForProductionDisassembly( int productionOrderDocEntry, bool areLogisticCarriersStoredOnOneLocation, int? currentProdBatchNumber, DbTool dbTool )
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

                strBuilder.Append( "SELECT " );
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.CustomItemDescr.NAME, PickItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.CodeBars.NAME, PickItemsForProductionDataSet.Columns.ITEMBARCODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME, PickItemsForProductionDataSet.Columns.ITEMCODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddIntQuantityColumnWithName(strBuilder, PickItemsForProductionDataSet.Columns.BASEQUANTITY.Name, 1);
                //no value but used as base qty? no WOR1.Baseqty / OWOR.BaseQty applied



                if (currentProdBatchNumber.HasValue)
                {
                    strBuilder.Append(", (( ");
                    strBuilder.AppendFormat(" ({0} / " + strBuilder.IFNULL + "( {1}, 1) ) * {2} ",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME),
                     SqlCommandHelper.FormatAndEscapeColumnValue(currentProdBatchNumber.Value, CultureInfo.InvariantCulture));

                }
                else
                {
                    strBuilder.Append(", ((");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);

                }

                strBuilder.Append(") - ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);

                strBuilder.Append(" - ", strBuilder.IFNULL, " ( SUM ( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
                strBuilder.Append(" ),0) ");


                strBuilder.Append(" ) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PickItemsForProductionDataSet.Columns.QUANTITY.Name));


                //strBuilder.Append(", (");
                //SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);
                //strBuilder.Append(" - ");
                //SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);

                //strBuilder.Append(" - ", strBuilder.IFNULL, " ( SUM ( ");
                //SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
                //strBuilder.Append(" ),0) ");


                //strBuilder.Append(" ) AS ");
                //strBuilder.Append(SqlCommandHelper.EscapeColumnName(PickItemsForProductionDataSet.Columns.QUANTITY.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UOM.NAME, PickItemsForProductionDataSet.Columns.UOM.Name);



                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);

                strBuilder.Append(" LEFT OUTER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);


                strBuilder.Append(" WHERE ");

                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

                if (areLogisticCarriersStoredOnOneLocation)
                {
                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));
                }

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.InventoryItem.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" GROUP BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.CustomItemDescr.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.CodeBars.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
                
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UOM.NAME);


                strBuilder.Append("HAVING (( ");
                if (currentProdBatchNumber.HasValue)
                {
                    strBuilder.AppendFormat(" ({0} / " + strBuilder.IFNULL + "( {1}, 1) ) * {2} ",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME),
                     SqlCommandHelper.FormatAndEscapeColumnValue(currentProdBatchNumber.Value, CultureInfo.InvariantCulture));

                }
                else
                {
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);
                }

                strBuilder.Append(") - ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);

                strBuilder.Append(" - ", strBuilder.IFNULL, " ( SUM ( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
                strBuilder.Append(" ),0) ) > 0");



                //strBuilder.Append(" HAVING (");
                //SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);
                //strBuilder.Append(" - ");
                //SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);
                //strBuilder.Append(" - ", strBuilder.IFNULL, " ( SUM ( ");
                //SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
                //strBuilder.Append(" ),0) ");
                //strBuilder.Append(" ) ");
                //strBuilder.Append(" > 0 ");

                strBuilder.Append(" ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);


                return strBuilder.ToString();
            }


            /// <summary>
            /// Gets the quantity already issued.
            /// </summary>
            /// <param name="productionOrderDocEntry">The production order doc entry.</param>
            /// <param name="currentProductionBatch">The current production batch.</param>
            /// <returns></returns>
            internal static string GetQuantityAlreadyIssued( int productionOrderDocEntry, int currentProductionBatch, DbTool dbTool )
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

                strBuilder.AppendFormat( "SELECT " + strBuilder.IFNULL + "(SUM({0}),0) AS {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.Quantity.NAME),
                     SqlCommandHelper.EscapeColumnName(PmxProductionReceiptLineTable.Columns.Quantity.NAME)
                    );

                strBuilder.AppendFormat("\r\n FROM {0}",
                    SqlCommandHelper.EscapeTableName(PmxProductionReceiptLineTable.NAME)
                 );

                strBuilder.AppendFormat("\r\n WHERE {0} = {1} AND {2} = {3}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.BaseEntry.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.ProductionBatch.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(currentProductionBatch, CultureInfo.InvariantCulture)
                );

                return strBuilder.ToString();

            }

            /// <summary>
            /// Gets the production orders.
            /// </summary>
            /// <param name="productionLineCode">The production line code.</param>
            /// <param name="productionOrderDocEntry">The production order doc entry.</param>
            /// <param name="dbTool">The db tool.</param>
            /// <returns></returns>
            public static string GetProductionOrders( string productionLineCode, int? productionOrderDocEntry, DbTool dbTool )
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

                strBuilder.Append( "SELECT " );
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductionOrderDataSet.Columns.ITEMDESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME, ProductionOrderDataSet.Columns.ITEMCODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME, ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name);

                strBuilder.Append(", CAST(");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
                strBuilder.Append(" AS NVARCHAR) AS ");
                strBuilder.Append(strBuilder.AsIdentifier(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY_AS_STRING.Name));

                strBuilder.Append(", CAST(");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME);
                strBuilder.Append(" AS NVARCHAR) AS ");
                strBuilder.Append(strBuilder.AsIdentifier(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_NUM_AS_STRING.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Comments.NAME, ProductionOrderDataSet.Columns.COMMENTS.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME, ProductionOrderDataSet.Columns.DOC_NUM.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME, ProductionOrderDataSet.Columns.PRODUCTION_LINE_CODE.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionOrderStepListKey.NAME, ProductionOrderDataSet.Columns.STEP_LIST_KEY.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME, ProductionOrderDataSet.Columns.DUE_DATE.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CardCode.NAME, ProductionOrderDataSet.Columns.CUSTOMER_CODE.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardName.NAME, ProductionOrderDataSet.Columns.CUSTOMER_DESCRIPTION.Name));

                strBuilder.Append(", (");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);
                strBuilder.Append(" - ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);
                strBuilder.Append(" - ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.RejectedQuantity.NAME);

                strBuilder.Append(" ) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductionOrderDataSet.Columns.OPEN_QUANTITY.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.InputLocationCode.NAME, ProductionOrderDataSet.Columns.INPUT_LOCATION_CODE.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.NAME, PmxItemTable.Columns.UOM.NAME, ProductionOrderDataSet.Columns.UOM.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Type.NAME, ProductionOrderDataSet.Columns.PRODUCTION_ORDER_TYPE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionStatus.NAME, ProductionOrderDataSet.Columns.PMX_PRODUCTION_STATUS.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.BatchNumber.NAME, ProductionOrderDataSet.Columns.PMX_BATCH_NUM.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProdBatchSize.NAME, PmxProductionOrderTable.Columns.ProdBatchSize.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);
                strBuilder.AppendFormat(", " + strBuilder.IFNULL + "({0}, 1) AS {1}",
                SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CurrentProductionBatch.NAME),
                SqlCommandHelper.FormatAndEscapeColumnValue(PmxProductionOrderTable.Columns.CurrentProductionBatch.NAME, CultureInfo.InvariantCulture)
                );

                strBuilder.Append("\r\n FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));


                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Warehouse.NAME);

                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.IsManagedByProdumex.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));



                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                strBuilder.Append("\r\n ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);

                strBuilder.Append("\r\n LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboBusinessPartnerTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CardCode.NAME);

                strBuilder.Append("\r\n LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.Code.NAME);



                strBuilder.Append("\r\n WHERE ");
                strBuilder.Append(" ( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME);
                strBuilder.Append(" IS NULL ");

                if (!string.IsNullOrEmpty(productionLineCode))
                {
                    strBuilder.Append(" OR ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME);
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionLineCode, CultureInfo.InvariantCulture));
                }
                strBuilder.Append(" ) ");

                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Status.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboProductionOrderStatusColumnValues.Released.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Type.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboProductionOrderTypeValues.Disassembly.VALUE, CultureInfo.InvariantCulture));


                if (productionOrderDocEntry.HasValue)
                {
                    strBuilder.Append("\r\n AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry.Value, CultureInfo.InvariantCulture));

                }


                strBuilder.Append("\r\n ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME);


                return strBuilder.ToString();
            }


            /// <summary>
            /// Gets the started production order for the Production line.
            /// </summary>
            /// <param name="productionLineCode">The production line code.</param>
            /// <returns></returns>
            public static string GetStartedProductionOrders(string productionLineCode)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductionOrderDataSet.Columns.ITEMDESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME, ProductionOrderDataSet.Columns.ITEMCODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME, ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name);

                strBuilder.Append(", CAST(");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
                strBuilder.Append(" AS NVARCHAR) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY_AS_STRING.Name));

                strBuilder.Append(", CAST(");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME);
                strBuilder.Append(" AS NVARCHAR) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_NUM_AS_STRING.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Comments.NAME, ProductionOrderDataSet.Columns.COMMENTS.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME, ProductionOrderDataSet.Columns.DOC_NUM.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME, ProductionOrderDataSet.Columns.PRODUCTION_LINE_CODE.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME, ProductionOrderDataSet.Columns.DUE_DATE.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CardCode.NAME, ProductionOrderDataSet.Columns.CUSTOMER_CODE.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardName.NAME, ProductionOrderDataSet.Columns.CUSTOMER_DESCRIPTION.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionOrderStepListKey.NAME, ProductionOrderDataSet.Columns.STEP_LIST_KEY.Name));

                strBuilder.Append(", (");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);
                strBuilder.Append(" - ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);
                strBuilder.Append(" - ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.RejectedQuantity.NAME);

                strBuilder.Append(" ) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductionOrderDataSet.Columns.OPEN_QUANTITY.Name));


                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.NAME, PmxItemTable.Columns.UOM.NAME, ProductionOrderDataSet.Columns.UOM.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Type.NAME, ProductionOrderDataSet.Columns.PRODUCTION_ORDER_TYPE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionStatus.NAME, ProductionOrderDataSet.Columns.PMX_PRODUCTION_STATUS.Name);


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboBusinessPartnerTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CardCode.NAME);


                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionLineCode, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Status.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboProductionOrderStatusColumnValues.Released.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionStatus.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxProductionStatusColumnValues.Started.VALUE, CultureInfo.InvariantCulture));



                strBuilder.Append(" ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME);


                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets all the production lines.
            /// </summary>
            /// <param name="warehouseCode">The warehouse code.</param>
            /// <returns>The sql statement to get all the production lines</returns>
            public static string GetProductionLines(string warehouseCode, string deviceID)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name );

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name );

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name );

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.InputLocationCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.OutputLocationCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.RestLocationCode.NAME));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME);

                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.IsManagedByProdumex.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));



                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.Code.NAME);



                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));



                if (!String.IsNullOrEmpty(warehouseCode) || !String.IsNullOrEmpty(deviceID))
                {
                    strBuilder.Append(" AND ( ");
                    if (!String.IsNullOrEmpty(warehouseCode))
                    {
                        SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                        strBuilder.Append(" = ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, CultureInfo.InvariantCulture));
                    }
                    if (!String.IsNullOrEmpty(deviceID))
                    {
                        if (!String.IsNullOrEmpty(warehouseCode)) strBuilder.Append(" OR ");

                        strBuilder.AppendFormat("{0} IN ( SELECT {1} FROM {2} WHERE {3} = {4} )",
                            SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
                            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.PmxWarehouseCode.NAME),
                            SqlCommandHelper.EscapeTableName(PmxThinClientWarehouseTable.NAME),
                            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.ParentCode.NAME),
                            SqlCommandHelper.FormatAndEscapeColumnValue(deviceID, CultureInfo.InvariantCulture));
                    }
                    strBuilder.Append(" )");
                }


                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME));


                return strBuilder.ToString();
            }

			internal static string GetExistingSSSCCsLinkedToProductionOrder(int productionOrderDocEntry, DbTool dbTool)
			{
				/*
				  select distinct PMX_LUID.SSCC
					from PMX_INVD
					inner join PMX_LUID on PMX_INVD.LogUnitIdentKey = PMX_LUID.InternalKey
					inner join PMX_INVT on PMX_INVT.LogUnitIdentKey = PMX_LUID.InternalKey
					where PMX_INVD.BaseType  = N'202' 
					and TransType = N'59'
					and PMX_INVD.BaseEntry = docEntry
				 */

				DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();


                strBuilder.AppendFormat("SELECT DISTINCT {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME, SSCCDataSet.Columns.SSCC.Name)
                    );

                strBuilder.AppendFormat(", {0}",
                     SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME, SSCCDataSet.Columns.LUID.Name)
                     );

                strBuilder.AppendFormat(", {0}",
                   SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name)
                   );


                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name));

                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name));


                strBuilder.AppendFormat("\r\n FROM {0}",
					SqlCommandHelper.EscapeTableName(PmxInventoryDetailTable.NAME)
				 );

				strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2}",
					SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.LogUnitIdentKey.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME));

				strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2}",
					SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME));


                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME));

                strBuilder.AppendFormat("\r\n WHERE {0} = {1} AND {2} = {3} AND {4} = {5}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.BaseType.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypeValues.ProductionOrder, CultureInfo.InvariantCulture),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.TransType.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(((int)SboObjectType.ProductionReceipt).ToString(), CultureInfo.InvariantCulture),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.BaseEntry.NAME),
					SqlCommandHelper.EscapeColumnValue(productionOrderDocEntry));

				return strBuilder.ToString();
			}
        }

        #endregion
    }
}
